# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing
from json.decoder import JSONDecodeError

from .......core.api_error import ApiError
from .......core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from .......core.datetime_utils import serialize_datetime
from .......core.http_response import AsyncHttpResponse, HttpResponse
from .......core.jsonable_encoder import jsonable_encoder
from .......core.pydantic_utilities import parse_obj_as
from .......core.request_options import RequestOptions
from ....common.errors.bad_request_error import BadRequestError
from ....common.errors.not_found_error import NotFoundError
from ....common.errors.version_conflict_error import VersionConflictError
from ....common.types.appointment_id import AppointmentId
from ....common.types.error_base_4_xx import ErrorBase4Xx
from ....common.types.filter_query_string import FilterQueryString
from ....common.types.page_token import PageToken
from ....common.types.sort_direction import SortDirection
from ....common.types.version_conflict_error_body import VersionConflictErrorBody
from ....lists.resources.v_1.types.sort_field_string import SortFieldString
from .types.appointment import Appointment
from .types.mutable_appointment import MutableAppointment
from .types.visits_page import VisitsPage

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawV1Client:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def create(
        self, *, request: MutableAppointment, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[Appointment]:
        """
        Adds an appointment.  VersionConflictError is returned when the placer_appointment_id is already in use.

        Parameters
        ----------
        request : MutableAppointment

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[Appointment]
        """
        _response = self._client_wrapper.httpx_client.request(
            "appointments/v1",
            base_url=self._client_wrapper.get_environment().pre_encounter,
            method="POST",
            json=request,
            request_options=request_options,
            omit=OMIT,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                Appointment,
                parse_obj_as(
                    type_=Appointment,  # type: ignore
                    object_=_response_json,
                ),
            )
            return HttpResponse(response=_response, data=_data)
        if "errorName" in _response_json:
            if _response_json["errorName"] == "VersionConflictError":
                raise VersionConflictError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        VersionConflictErrorBody,
                        parse_obj_as(
                            type_=VersionConflictErrorBody,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "NotFoundError":
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorBase4Xx,
                        parse_obj_as(
                            type_=ErrorBase4Xx,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def get_visits(
        self,
        *,
        page_token: typing.Optional[PageToken] = None,
        limit: typing.Optional[int] = None,
        sort_field: typing.Optional[SortFieldString] = None,
        sort_direction: typing.Optional[SortDirection] = None,
        filters: typing.Optional[FilterQueryString] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[VisitsPage]:
        """
        Gets all Visits within a given time range. The return list is ordered by start_time ascending.

        **IMPORTANT:** This endpoint requires a date filter on `appointment.startTimestamp` to ensure acceptable query performance.
        Without date filtering, the query can take 50+ seconds on large datasets due to grouping and aggregation operations.

        Example filters:
        - `appointment.startTimestamp|gt|2024-01-01` - appointments after January 1, 2024
        - `appointment.startTimestamp|eq|2024-12-08` - appointments on December 8, 2024
        - `appointment.startTimestamp|lt|2024-12-31` - appointments before December 31, 2024

        You can combine the date filter with other filters using commas:
        - `appointment.startTimestamp|gt|2024-01-01,appointment.status|eq|PENDING`

        Parameters
        ----------
        page_token : typing.Optional[PageToken]

        limit : typing.Optional[int]

        sort_field : typing.Optional[SortFieldString]
            Defaults to appointment.start_time.

        sort_direction : typing.Optional[SortDirection]
            Defaults to ascending.

        filters : typing.Optional[FilterQueryString]
            **Required:** Must include a date filter on appointment.startTimestamp (using gt, lt, or eq operators).
            Example: appointment.startTimestamp|gt|2024-01-01

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[VisitsPage]
        """
        _response = self._client_wrapper.httpx_client.request(
            "appointments/v1/visits",
            base_url=self._client_wrapper.get_environment().pre_encounter,
            method="GET",
            params={
                "page_token": page_token,
                "limit": limit,
                "sort_field": sort_field,
                "sort_direction": sort_direction,
                "filters": filters,
            },
            request_options=request_options,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                VisitsPage,
                parse_obj_as(
                    type_=VisitsPage,  # type: ignore
                    object_=_response_json,
                ),
            )
            return HttpResponse(response=_response, data=_data)
        if "errorName" in _response_json:
            if _response_json["errorName"] == "BadRequestError":
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorBase4Xx,
                        parse_obj_as(
                            type_=ErrorBase4Xx,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def get(
        self, id: AppointmentId, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[Appointment]:
        """
        Gets an appointment.

        Parameters
        ----------
        id : AppointmentId

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[Appointment]
        """
        _response = self._client_wrapper.httpx_client.request(
            f"appointments/v1/{jsonable_encoder(id)}",
            base_url=self._client_wrapper.get_environment().pre_encounter,
            method="GET",
            request_options=request_options,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                Appointment,
                parse_obj_as(
                    type_=Appointment,  # type: ignore
                    object_=_response_json,
                ),
            )
            return HttpResponse(response=_response, data=_data)
        if "errorName" in _response_json:
            if _response_json["errorName"] == "NotFoundError":
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorBase4Xx,
                        parse_obj_as(
                            type_=ErrorBase4Xx,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def get_history(
        self, id: AppointmentId, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[typing.List[Appointment]]:
        """
        Gets an appointment along with it's full history.  The return list is ordered by version ascending.

        Parameters
        ----------
        id : AppointmentId

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[typing.List[Appointment]]
        """
        _response = self._client_wrapper.httpx_client.request(
            f"appointments/v1/{jsonable_encoder(id)}/history",
            base_url=self._client_wrapper.get_environment().pre_encounter,
            method="GET",
            request_options=request_options,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                typing.List[Appointment],
                parse_obj_as(
                    type_=typing.List[Appointment],  # type: ignore
                    object_=_response_json,
                ),
            )
            return HttpResponse(response=_response, data=_data)
        if "errorName" in _response_json:
            if _response_json["errorName"] == "NotFoundError":
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorBase4Xx,
                        parse_obj_as(
                            type_=ErrorBase4Xx,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def update(
        self,
        id: AppointmentId,
        version: str,
        *,
        request: MutableAppointment,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[Appointment]:
        """
        Updates an appointment. The path must contain the next version number to prevent race conditions. For example, if the current version of the appointment is n, you will need to send a request to this endpoint with `/{id}/n+1` to update the appointment. Updating historic versions is not supported.

        Parameters
        ----------
        id : AppointmentId

        version : str

        request : MutableAppointment

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[Appointment]
        """
        _response = self._client_wrapper.httpx_client.request(
            f"appointments/v1/{jsonable_encoder(id)}/{jsonable_encoder(version)}",
            base_url=self._client_wrapper.get_environment().pre_encounter,
            method="PUT",
            json=request,
            request_options=request_options,
            omit=OMIT,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                Appointment,
                parse_obj_as(
                    type_=Appointment,  # type: ignore
                    object_=_response_json,
                ),
            )
            return HttpResponse(response=_response, data=_data)
        if "errorName" in _response_json:
            if _response_json["errorName"] == "NotFoundError":
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorBase4Xx,
                        parse_obj_as(
                            type_=ErrorBase4Xx,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "VersionConflictError":
                raise VersionConflictError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        VersionConflictErrorBody,
                        parse_obj_as(
                            type_=VersionConflictErrorBody,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def scan(
        self, *, since: dt.datetime, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[typing.List[Appointment]]:
        """
        Scans up to 100 appointment updates.  The since query parameter is inclusive, and the result list is ordered by updatedAt ascending.

        Parameters
        ----------
        since : dt.datetime

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[typing.List[Appointment]]
        """
        _response = self._client_wrapper.httpx_client.request(
            "appointments/v1/updates/scan",
            base_url=self._client_wrapper.get_environment().pre_encounter,
            method="GET",
            params={
                "since": serialize_datetime(since),
            },
            request_options=request_options,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                typing.List[Appointment],
                parse_obj_as(
                    type_=typing.List[Appointment],  # type: ignore
                    object_=_response_json,
                ),
            )
            return HttpResponse(response=_response, data=_data)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def deactivate(
        self, id: AppointmentId, version: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[None]:
        """
        Sets an appointment as deactivated.  The path must contain the most recent version to prevent race conditions.  Deactivating historic versions is not supported. Subsequent updates via PUT to the appointment will "reactivate" the appointment and set the deactivated flag to false.

        Parameters
        ----------
        id : AppointmentId

        version : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[None]
        """
        _response = self._client_wrapper.httpx_client.request(
            f"appointments/v1/{jsonable_encoder(id)}/{jsonable_encoder(version)}",
            base_url=self._client_wrapper.get_environment().pre_encounter,
            method="DELETE",
            request_options=request_options,
        )
        if 200 <= _response.status_code < 300:
            return HttpResponse(response=_response, data=None)
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if "errorName" in _response_json:
            if _response_json["errorName"] == "NotFoundError":
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorBase4Xx,
                        parse_obj_as(
                            type_=ErrorBase4Xx,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "VersionConflictError":
                raise VersionConflictError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        VersionConflictErrorBody,
                        parse_obj_as(
                            type_=VersionConflictErrorBody,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawV1Client:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def create(
        self, *, request: MutableAppointment, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[Appointment]:
        """
        Adds an appointment.  VersionConflictError is returned when the placer_appointment_id is already in use.

        Parameters
        ----------
        request : MutableAppointment

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[Appointment]
        """
        _response = await self._client_wrapper.httpx_client.request(
            "appointments/v1",
            base_url=self._client_wrapper.get_environment().pre_encounter,
            method="POST",
            json=request,
            request_options=request_options,
            omit=OMIT,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                Appointment,
                parse_obj_as(
                    type_=Appointment,  # type: ignore
                    object_=_response_json,
                ),
            )
            return AsyncHttpResponse(response=_response, data=_data)
        if "errorName" in _response_json:
            if _response_json["errorName"] == "VersionConflictError":
                raise VersionConflictError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        VersionConflictErrorBody,
                        parse_obj_as(
                            type_=VersionConflictErrorBody,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "NotFoundError":
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorBase4Xx,
                        parse_obj_as(
                            type_=ErrorBase4Xx,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def get_visits(
        self,
        *,
        page_token: typing.Optional[PageToken] = None,
        limit: typing.Optional[int] = None,
        sort_field: typing.Optional[SortFieldString] = None,
        sort_direction: typing.Optional[SortDirection] = None,
        filters: typing.Optional[FilterQueryString] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[VisitsPage]:
        """
        Gets all Visits within a given time range. The return list is ordered by start_time ascending.

        **IMPORTANT:** This endpoint requires a date filter on `appointment.startTimestamp` to ensure acceptable query performance.
        Without date filtering, the query can take 50+ seconds on large datasets due to grouping and aggregation operations.

        Example filters:
        - `appointment.startTimestamp|gt|2024-01-01` - appointments after January 1, 2024
        - `appointment.startTimestamp|eq|2024-12-08` - appointments on December 8, 2024
        - `appointment.startTimestamp|lt|2024-12-31` - appointments before December 31, 2024

        You can combine the date filter with other filters using commas:
        - `appointment.startTimestamp|gt|2024-01-01,appointment.status|eq|PENDING`

        Parameters
        ----------
        page_token : typing.Optional[PageToken]

        limit : typing.Optional[int]

        sort_field : typing.Optional[SortFieldString]
            Defaults to appointment.start_time.

        sort_direction : typing.Optional[SortDirection]
            Defaults to ascending.

        filters : typing.Optional[FilterQueryString]
            **Required:** Must include a date filter on appointment.startTimestamp (using gt, lt, or eq operators).
            Example: appointment.startTimestamp|gt|2024-01-01

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[VisitsPage]
        """
        _response = await self._client_wrapper.httpx_client.request(
            "appointments/v1/visits",
            base_url=self._client_wrapper.get_environment().pre_encounter,
            method="GET",
            params={
                "page_token": page_token,
                "limit": limit,
                "sort_field": sort_field,
                "sort_direction": sort_direction,
                "filters": filters,
            },
            request_options=request_options,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                VisitsPage,
                parse_obj_as(
                    type_=VisitsPage,  # type: ignore
                    object_=_response_json,
                ),
            )
            return AsyncHttpResponse(response=_response, data=_data)
        if "errorName" in _response_json:
            if _response_json["errorName"] == "BadRequestError":
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorBase4Xx,
                        parse_obj_as(
                            type_=ErrorBase4Xx,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def get(
        self, id: AppointmentId, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[Appointment]:
        """
        Gets an appointment.

        Parameters
        ----------
        id : AppointmentId

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[Appointment]
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"appointments/v1/{jsonable_encoder(id)}",
            base_url=self._client_wrapper.get_environment().pre_encounter,
            method="GET",
            request_options=request_options,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                Appointment,
                parse_obj_as(
                    type_=Appointment,  # type: ignore
                    object_=_response_json,
                ),
            )
            return AsyncHttpResponse(response=_response, data=_data)
        if "errorName" in _response_json:
            if _response_json["errorName"] == "NotFoundError":
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorBase4Xx,
                        parse_obj_as(
                            type_=ErrorBase4Xx,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def get_history(
        self, id: AppointmentId, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[typing.List[Appointment]]:
        """
        Gets an appointment along with it's full history.  The return list is ordered by version ascending.

        Parameters
        ----------
        id : AppointmentId

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[typing.List[Appointment]]
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"appointments/v1/{jsonable_encoder(id)}/history",
            base_url=self._client_wrapper.get_environment().pre_encounter,
            method="GET",
            request_options=request_options,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                typing.List[Appointment],
                parse_obj_as(
                    type_=typing.List[Appointment],  # type: ignore
                    object_=_response_json,
                ),
            )
            return AsyncHttpResponse(response=_response, data=_data)
        if "errorName" in _response_json:
            if _response_json["errorName"] == "NotFoundError":
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorBase4Xx,
                        parse_obj_as(
                            type_=ErrorBase4Xx,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def update(
        self,
        id: AppointmentId,
        version: str,
        *,
        request: MutableAppointment,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[Appointment]:
        """
        Updates an appointment. The path must contain the next version number to prevent race conditions. For example, if the current version of the appointment is n, you will need to send a request to this endpoint with `/{id}/n+1` to update the appointment. Updating historic versions is not supported.

        Parameters
        ----------
        id : AppointmentId

        version : str

        request : MutableAppointment

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[Appointment]
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"appointments/v1/{jsonable_encoder(id)}/{jsonable_encoder(version)}",
            base_url=self._client_wrapper.get_environment().pre_encounter,
            method="PUT",
            json=request,
            request_options=request_options,
            omit=OMIT,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                Appointment,
                parse_obj_as(
                    type_=Appointment,  # type: ignore
                    object_=_response_json,
                ),
            )
            return AsyncHttpResponse(response=_response, data=_data)
        if "errorName" in _response_json:
            if _response_json["errorName"] == "NotFoundError":
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorBase4Xx,
                        parse_obj_as(
                            type_=ErrorBase4Xx,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "VersionConflictError":
                raise VersionConflictError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        VersionConflictErrorBody,
                        parse_obj_as(
                            type_=VersionConflictErrorBody,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def scan(
        self, *, since: dt.datetime, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[typing.List[Appointment]]:
        """
        Scans up to 100 appointment updates.  The since query parameter is inclusive, and the result list is ordered by updatedAt ascending.

        Parameters
        ----------
        since : dt.datetime

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[typing.List[Appointment]]
        """
        _response = await self._client_wrapper.httpx_client.request(
            "appointments/v1/updates/scan",
            base_url=self._client_wrapper.get_environment().pre_encounter,
            method="GET",
            params={
                "since": serialize_datetime(since),
            },
            request_options=request_options,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                typing.List[Appointment],
                parse_obj_as(
                    type_=typing.List[Appointment],  # type: ignore
                    object_=_response_json,
                ),
            )
            return AsyncHttpResponse(response=_response, data=_data)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def deactivate(
        self, id: AppointmentId, version: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[None]:
        """
        Sets an appointment as deactivated.  The path must contain the most recent version to prevent race conditions.  Deactivating historic versions is not supported. Subsequent updates via PUT to the appointment will "reactivate" the appointment and set the deactivated flag to false.

        Parameters
        ----------
        id : AppointmentId

        version : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[None]
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"appointments/v1/{jsonable_encoder(id)}/{jsonable_encoder(version)}",
            base_url=self._client_wrapper.get_environment().pre_encounter,
            method="DELETE",
            request_options=request_options,
        )
        if 200 <= _response.status_code < 300:
            return AsyncHttpResponse(response=_response, data=None)
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if "errorName" in _response_json:
            if _response_json["errorName"] == "NotFoundError":
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorBase4Xx,
                        parse_obj_as(
                            type_=ErrorBase4Xx,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "VersionConflictError":
                raise VersionConflictError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        VersionConflictErrorBody,
                        parse_obj_as(
                            type_=VersionConflictErrorBody,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing

from ......core import enum

T_Result = typing.TypeVar("T_Result")


class NonInsurancePayerSortField(enum.StrEnum):
    NAME = "NAME"
    CATEGORY = "CATEGORY"
    ENABLED = "ENABLED"
    UPDATED_AT = "UPDATED_AT"
    _UNKNOWN = "__NONINSURANCEPAYERSORTFIELD_UNKNOWN__"
    """
    This member is used for forward compatibility. If the value is not recognized by the enum, it will be stored here, and the raw value is accessible through `.value`.
    """

    @classmethod
    def _missing_(cls, value: typing.Any) -> "NonInsurancePayerSortField":
        unknown = cls._UNKNOWN
        unknown._value_ = value
        return unknown

    def visit(
        self,
        name: typing.Callable[[], T_Result],
        category: typing.Callable[[], T_Result],
        enabled: typing.Callable[[], T_Result],
        updated_at: typing.Callable[[], T_Result],
        _unknown_member: typing.Callable[[str], T_Result],
    ) -> T_Result:
        if self is NonInsurancePayerSortField.NAME:
            return name()
        if self is NonInsurancePayerSortField.CATEGORY:
            return category()
        if self is NonInsurancePayerSortField.ENABLED:
            return enabled()
        if self is NonInsurancePayerSortField.UPDATED_AT:
            return updated_at()
        return _unknown_member(self._value_)



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ....core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ...commons.types.removable_date_range_optional_end import RemovableDateRangeOptionalEnd
from .identifier_code import IdentifierCode
from .identifier_id import IdentifierId
from .identifier_value import IdentifierValue


class IdentifierUpdate(UniversalBaseModel):
    identifier_id: IdentifierId
    identifier_code: typing.Optional[IdentifierCode] = None
    identifier_value: typing.Optional[IdentifierValue] = None
    period: typing.Optional[RemovableDateRangeOptionalEnd] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from ......core import enum

T_Result = typing.TypeVar("T_Result")


class MeasurementUnitCode(enum.StrEnum):
    MILLILITERS = "ML"
    UNITS = "UN"
    GRAMS = "GR"
    INTERNATIONAL_UNIT = "F2"
    MILLIGRAM = "ME"
    _UNKNOWN = "__MEASUREMENTUNITCODE_UNKNOWN__"
    """
    This member is used for forward compatibility. If the value is not recognized by the enum, it will be stored here, and the raw value is accessible through `.value`.
    """

    @classmethod
    def _missing_(cls, value: typing.Any) -> "MeasurementUnitCode":
        unknown = cls._UNKNOWN
        unknown._value_ = value
        return unknown

    def visit(
        self,
        milliliters: typing.Callable[[], T_Result],
        units: typing.Callable[[], T_Result],
        grams: typing.Callable[[], T_Result],
        international_unit: typing.Callable[[], T_Result],
        milligram: typing.Callable[[], T_Result],
        _unknown_member: typing.Callable[[str], T_Result],
    ) -> T_Result:
        if self is MeasurementUnitCode.MILLILITERS:
            return milliliters()
        if self is MeasurementUnitCode.UNITS:
            return units()
        if self is MeasurementUnitCode.GRAMS:
            return grams()
        if self is MeasurementUnitCode.INTERNATIONAL_UNIT:
            return international_unit()
        if self is MeasurementUnitCode.MILLIGRAM:
            return milligram()
        return _unknown_member(self._value_)



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
from ....core.pydantic_utilities import IS_PYDANTIC_V2
from ...billing_notes.resources.v_2.types.billing_note_base import BillingNoteBase
from ...claim_submission.resources.v_1.types.external_claim_submission_create import ExternalClaimSubmissionCreate
from ...commons.types.pre_encounter_appointment_id import PreEncounterAppointmentId
from ...commons.types.pre_encounter_patient_id import PreEncounterPatientId
from ...custom_schemas.resources.v_1.types.schema_instance import SchemaInstance
from ...encounter_providers.resources.v_2.types.billing_provider import BillingProvider
from ...encounter_providers.resources.v_2.types.initial_referring_provider import InitialReferringProvider
from ...encounter_providers.resources.v_2.types.supervising_provider import SupervisingProvider
from ...encounters.resources.v_4.types.clinical_note_category_create import ClinicalNoteCategoryCreate
from ...encounters.resources.v_4.types.encounter_base import EncounterBase
from ...encounters.resources.v_4.types.patient_history_category import PatientHistoryCategory
from ...property_and_casualty.resources.v_1.types.property_casualty_patient_identifier_create import (
    PropertyCasualtyPatientIdentifierCreate,
)
from ...related_causes.resources.v_1.types.related_causes_information_create import RelatedCausesInformationCreate
from ...service_facility.types.encounter_service_facility_base import EncounterServiceFacilityBase
from ...tags.types.tag_id import TagId


class UniversalEncounterCreateFromPreEncounterBase(EncounterBase):
    pre_encounter_patient_id: PreEncounterPatientId
    pre_encounter_appointment_ids: typing.List[PreEncounterAppointmentId]
    billing_provider: BillingProvider = pydantic.Field()
    """
    The billing provider is the provider or business entity submitting the claim. Billing provider may be, but is not necessarily, the same person/NPI as the rendering provider. From a payer's perspective, this represents the person or entity being reimbursed. When a contract exists with the target payer, the billing provider should be the entity contracted with the payer. In some circumstances, this will be an individual provider. In that case, submit that provider's NPI and the tax ID (TIN) that the provider gave to the payer during contracting. In other cases, the billing entity will be a medical group. If so, submit the group NPI and the group's tax ID. Box 33 on the CMS-1500 claim or Form Locator 1 on a UB-04 claim form.
    """

    initial_referring_provider: typing.Optional[InitialReferringProvider] = pydantic.Field(default=None)
    """
    The second iteration of Loop ID-2310 on an 837P form. Use code "P3 - Primary Care Provider" in this loop to
    indicate the initial referral from the primary care provider or whatever provider wrote the initial referral for this patient's episode of care being billed/reported in this transaction.
    Not used in an 837i claim
    """

    supervising_provider: typing.Optional[SupervisingProvider] = pydantic.Field(default=None)
    """
    Required when the rendering provider is supervised by a physician. If not required by this implementation guide, do not send.
    """

    service_facility: typing.Optional[EncounterServiceFacilityBase] = pydantic.Field(default=None)
    """
    Encounter Service facility is typically the location a medical service was rendered, such as a provider office or hospital. For telehealth, service facility can represent the provider's location when the service was delivered (e.g., home), or the location where an in-person visit would have taken place, whichever is easier to identify. If the provider is in-network, service facility may be defined in payer contracts. Box 32 on the CMS-1500 claim form. There is no equivalent on the paper UB-04 claim form, but this field is equivalent to Loop 2310E Service Facility Location details on an 837i form, and is used when this is different to the entity identified as the Billing Provider. Note that for an in-network claim to be successfully adjudicated, the service facility address listed
    """

    clinical_notes: typing.Optional[typing.List[ClinicalNoteCategoryCreate]] = pydantic.Field(default=None)
    """
    Holds a collection of clinical observations made by healthcare providers during patient encounters. Please note that medical records for appeals should be sent using the Encounter Attachments API.
    """

    billing_notes: typing.Optional[typing.List[BillingNoteBase]] = pydantic.Field(default=None)
    """
    Spot to store misc, human-readable, notes about this encounter to be used
    in the billing process.
    """

    patient_histories: typing.Optional[typing.List[PatientHistoryCategory]] = None
    external_claim_submission: typing.Optional[ExternalClaimSubmissionCreate] = pydantic.Field(default=None)
    """
    To be included for claims that have been submitted outside of Candid.
    Candid supports posting remits and payments to these claims and working them in-platform (e.g. editing, resubmitting).
    """

    tag_ids: typing.Optional[typing.List[TagId]] = pydantic.Field(default=None)
    """
    Names of tags that should be on the encounter.
    """

    schema_instances: typing.Optional[typing.List[SchemaInstance]] = pydantic.Field(default=None)
    """
    Key-value pairs that must adhere to a schema created via the Custom Schema API. Multiple schema
    instances cannot be created for the same schema on an encounter.
    """

    related_causes_information: typing.Optional[RelatedCausesInformationCreate] = pydantic.Field(default=None)
    """
    Corresponds to box 10a on the CMS-1500 (Loop 2300 on 837)
    """

    property_casualty_claim_number: typing.Optional[str] = pydantic.Field(default=None)
    """
    837p Loop2010 REF02, CMS1500 Box 11b
    """

    accident_date: typing.Optional[dt.date] = pydantic.Field(default=None)
    """
    837p Loop2300 DTP*439, CMS1500 Box 15
    """

    property_casualty_patient_identifier: typing.Optional[PropertyCasualtyPatientIdentifierCreate] = pydantic.Field(
        default=None
    )
    """
    Patient identifier for Property and Casualty claims
    837p Loop 2010CA
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from ......core import enum

T_Result = typing.TypeVar("T_Result")


class EncounterAttachmentType(enum.StrEnum):
    DOCUMENTATION = "DOCUMENTATION"
    EOB = "EOB"
    OTHER = "OTHER"
    _UNKNOWN = "__ENCOUNTERATTACHMENTTYPE_UNKNOWN__"
    """
    This member is used for forward compatibility. If the value is not recognized by the enum, it will be stored here, and the raw value is accessible through `.value`.
    """

    @classmethod
    def _missing_(cls, value: typing.Any) -> "EncounterAttachmentType":
        unknown = cls._UNKNOWN
        unknown._value_ = value
        return unknown

    def visit(
        self,
        documentation: typing.Callable[[], T_Result],
        eob: typing.Callable[[], T_Result],
        other: typing.Callable[[], T_Result],
        _unknown_member: typing.Callable[[str], T_Result],
    ) -> T_Result:
        if self is EncounterAttachmentType.DOCUMENTATION:
            return documentation()
        if self is EncounterAttachmentType.EOB:
            return eob()
        if self is EncounterAttachmentType.OTHER:
            return other()
        return _unknown_member(self._value_)



# This file was auto-generated by Fern from our API Definition.

import typing

from .....core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from .....core.request_options import RequestOptions
from ....commons.types.service_line_id import ServiceLineId
from .raw_client import AsyncRawV2Client, RawV2Client
from .types.service_line import ServiceLine
from .types.service_line_create_standalone import ServiceLineCreateStandalone
from .types.service_line_update import ServiceLineUpdate
from .types.universal_service_line_create_standalone import UniversalServiceLineCreateStandalone
from .types.universal_service_line_update import UniversalServiceLineUpdate

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class V2Client:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawV2Client(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawV2Client:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawV2Client
        """
        return self._raw_client

    def create(
        self, *, request: ServiceLineCreateStandalone, request_options: typing.Optional[RequestOptions] = None
    ) -> ServiceLine:
        """
        Parameters
        ----------
        request : ServiceLineCreateStandalone

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ServiceLine

        Examples
        --------
        import uuid

        from candid import CandidApiClient
        from candid.resources.commons import ServiceLineUnits
        from candid.resources.service_lines.resources.v_2 import (
            ServiceLineCreateStandalone,
        )

        client = CandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        client.service_lines.v_2.create(
            request=ServiceLineCreateStandalone(
                procedure_code="procedure_code",
                quantity="quantity",
                units=ServiceLineUnits.MJ,
                claim_id=uuid.UUID(
                    "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
                ),
            ),
        )
        """
        _response = self._raw_client.create(request=request, request_options=request_options)
        return _response.data

    def create_universal(
        self, *, request: UniversalServiceLineCreateStandalone, request_options: typing.Optional[RequestOptions] = None
    ) -> ServiceLine:
        """
        Parameters
        ----------
        request : UniversalServiceLineCreateStandalone

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ServiceLine

        Examples
        --------
        import uuid

        from candid import CandidApiClient
        from candid.resources.commons import ServiceLineUnits
        from candid.resources.service_lines.resources.v_2 import (
            UniversalServiceLineCreateStandalone,
        )

        client = CandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        client.service_lines.v_2.create_universal(
            request=UniversalServiceLineCreateStandalone(
                quantity="quantity",
                units=ServiceLineUnits.MJ,
                claim_id=uuid.UUID(
                    "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
                ),
            ),
        )
        """
        _response = self._raw_client.create_universal(request=request, request_options=request_options)
        return _response.data

    def update_universal(
        self,
        service_line_id: ServiceLineId,
        *,
        request: UniversalServiceLineUpdate,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ServiceLine:
        """
        Parameters
        ----------
        service_line_id : ServiceLineId

        request : UniversalServiceLineUpdate

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ServiceLine

        Examples
        --------
        import uuid

        from candid import CandidApiClient
        from candid.resources.service_lines.resources.v_2 import (
            UniversalServiceLineUpdate,
        )

        client = CandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        client.service_lines.v_2.update_universal(
            service_line_id=uuid.UUID(
                "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
            ),
            request=UniversalServiceLineUpdate(),
        )
        """
        _response = self._raw_client.update_universal(service_line_id, request=request, request_options=request_options)
        return _response.data

    def update(
        self,
        service_line_id: ServiceLineId,
        *,
        request: ServiceLineUpdate,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ServiceLine:
        """
        Parameters
        ----------
        service_line_id : ServiceLineId

        request : ServiceLineUpdate

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ServiceLine

        Examples
        --------
        import uuid

        from candid import CandidApiClient
        from candid.resources.service_lines.resources.v_2 import ServiceLineUpdate

        client = CandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        client.service_lines.v_2.update(
            service_line_id=uuid.UUID(
                "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
            ),
            request=ServiceLineUpdate(),
        )
        """
        _response = self._raw_client.update(service_line_id, request=request, request_options=request_options)
        return _response.data

    def delete(
        self, service_line_id: ServiceLineId, *, request_options: typing.Optional[RequestOptions] = None
    ) -> None:
        """
        Parameters
        ----------
        service_line_id : ServiceLineId

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import uuid

        from candid import CandidApiClient

        client = CandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        client.service_lines.v_2.delete(
            service_line_id=uuid.UUID(
                "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
            ),
        )
        """
        _response = self._raw_client.delete(service_line_id, request_options=request_options)
        return _response.data


class AsyncV2Client:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawV2Client(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawV2Client:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawV2Client
        """
        return self._raw_client

    async def create(
        self, *, request: ServiceLineCreateStandalone, request_options: typing.Optional[RequestOptions] = None
    ) -> ServiceLine:
        """
        Parameters
        ----------
        request : ServiceLineCreateStandalone

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ServiceLine

        Examples
        --------
        import asyncio
        import uuid

        from candid import AsyncCandidApiClient
        from candid.resources.commons import ServiceLineUnits
        from candid.resources.service_lines.resources.v_2 import (
            ServiceLineCreateStandalone,
        )

        client = AsyncCandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )


        async def main() -> None:
            await client.service_lines.v_2.create(
                request=ServiceLineCreateStandalone(
                    procedure_code="procedure_code",
                    quantity="quantity",
                    units=ServiceLineUnits.MJ,
                    claim_id=uuid.UUID(
                        "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
                    ),
                ),
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.create(request=request, request_options=request_options)
        return _response.data

    async def create_universal(
        self, *, request: UniversalServiceLineCreateStandalone, request_options: typing.Optional[RequestOptions] = None
    ) -> ServiceLine:
        """
        Parameters
        ----------
        request : UniversalServiceLineCreateStandalone

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ServiceLine

        Examples
        --------
        import asyncio
        import uuid

        from candid import AsyncCandidApiClient
        from candid.resources.commons import ServiceLineUnits
        from candid.resources.service_lines.resources.v_2 import (
            UniversalServiceLineCreateStandalone,
        )

        client = AsyncCandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )


        async def main() -> None:
            await client.service_lines.v_2.create_universal(
                request=UniversalServiceLineCreateStandalone(
                    quantity="quantity",
                    units=ServiceLineUnits.MJ,
                    claim_id=uuid.UUID(
                        "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
                    ),
                ),
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.create_universal(request=request, request_options=request_options)
        return _response.data

    async def update_universal(
        self,
        service_line_id: ServiceLineId,
        *,
        request: UniversalServiceLineUpdate,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ServiceLine:
        """
        Parameters
        ----------
        service_line_id : ServiceLineId

        request : UniversalServiceLineUpdate

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ServiceLine

        Examples
        --------
        import asyncio
        import uuid

        from candid import AsyncCandidApiClient
        from candid.resources.service_lines.resources.v_2 import (
            UniversalServiceLineUpdate,
        )

        client = AsyncCandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )


        async def main() -> None:
            await client.service_lines.v_2.update_universal(
                service_line_id=uuid.UUID(
                    "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
                ),
                request=UniversalServiceLineUpdate(),
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.update_universal(
            service_line_id, request=request, request_options=request_options
        )
        return _response.data

    async def update(
        self,
        service_line_id: ServiceLineId,
        *,
        request: ServiceLineUpdate,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ServiceLine:
        """
        Parameters
        ----------
        service_line_id : ServiceLineId

        request : ServiceLineUpdate

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ServiceLine

        Examples
        --------
        import asyncio
        import uuid

        from candid import AsyncCandidApiClient
        from candid.resources.service_lines.resources.v_2 import ServiceLineUpdate

        client = AsyncCandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )


        async def main() -> None:
            await client.service_lines.v_2.update(
                service_line_id=uuid.UUID(
                    "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
                ),
                request=ServiceLineUpdate(),
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.update(service_line_id, request=request, request_options=request_options)
        return _response.data

    async def delete(
        self, service_line_id: ServiceLineId, *, request_options: typing.Optional[RequestOptions] = None
    ) -> None:
        """
        Parameters
        ----------
        service_line_id : ServiceLineId

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio
        import uuid

        from candid import AsyncCandidApiClient

        client = AsyncCandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )


        async def main() -> None:
            await client.service_lines.v_2.delete(
                service_line_id=uuid.UUID(
                    "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
                ),
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.delete(service_line_id, request_options=request_options)
        return _response.data



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

import typing
from importlib import import_module

if typing.TYPE_CHECKING:
    from .adjustment_id import AdjustmentId
    from .appointment_id import AppointmentId
    from .bad_request_error_message import BadRequestErrorMessage
    from .billing_provider_commercial_license_type import BillingProviderCommercialLicenseType
    from .charge_capture_claim_creation_id import ChargeCaptureClaimCreationId
    from .charge_capture_id import ChargeCaptureId
    from .charge_capture_post_billed_change_id import ChargeCapturePostBilledChangeId
    from .claim_adjustment_group_codes import ClaimAdjustmentGroupCodes
    from .claim_id import ClaimId
    from .claim_submission_payer_responsibility_type import ClaimSubmissionPayerResponsibilityType
    from .clearinghouse import Clearinghouse
    from .clinical_trial_id import ClinicalTrialId
    from .date import Date
    from .date_range_optional_end import DateRangeOptionalEnd
    from .decimal import Decimal
    from .delay_reason_code import DelayReasonCode
    from .email import Email
    from .emr_payer_crosswalk import EmrPayerCrosswalk
    from .encounter_external_id import EncounterExternalId
    from .encounter_id import EncounterId
    from .entity_conflict_error_message import EntityConflictErrorMessage
    from .entity_not_found_error_message import EntityNotFoundErrorMessage
    from .epsdt_referral_condition_indicator_code import EpsdtReferralConditionIndicatorCode
    from .error_message import ErrorMessage
    from .facility_type_code import FacilityTypeCode
    from .insurance_type_code import InsuranceTypeCode
    from .intended_submission_medium import IntendedSubmissionMedium
    from .internal_error_message import InternalErrorMessage
    from .invoice_id import InvoiceId
    from .link_url import LinkUrl
    from .network_type import NetworkType
    from .next_responsible_party import NextResponsibleParty
    from .npi import Npi
    from .organization_id import OrganizationId
    from .organization_not_authorized_error_message import OrganizationNotAuthorizedErrorMessage
    from .page_token import PageToken
    from .patient_external_id import PatientExternalId
    from .patient_relationship_to_insured_code_all import PatientRelationshipToInsuredCodeAll
    from .payer_plan_group_id import PayerPlanGroupId
    from .phone_number import PhoneNumber
    from .phone_number_optional import PhoneNumberOptional
    from .phone_number_type import PhoneNumberType
    from .pre_encounter_appointment_id import PreEncounterAppointmentId
    from .pre_encounter_patient_id import PreEncounterPatientId
    from .primitive import Primitive
    from .procedure_modifier import ProcedureModifier
    from .provider_credentialing_span_id import ProviderCredentialingSpanId
    from .provider_id import ProviderId
    from .qualifier_code import QualifierCode
    from .rate_id import RateId
    from .region_national import RegionNational
    from .region_states import RegionStates
    from .regions import Regions, Regions_National, Regions_States
    from .removable_date_range_optional_end import (
        RemovableDateRangeOptionalEnd,
        RemovableDateRangeOptionalEnd_DateRange,
        RemovableDateRangeOptionalEnd_Remove,
    )
    from .report_transmission_code import ReportTransmissionCode
    from .report_type_code import ReportTypeCode
    from .request_validation_error import RequestValidationError
    from .resource_page import ResourcePage
    from .schema_id import SchemaId
    from .service_line_id import ServiceLineId
    from .service_line_units import ServiceLineUnits
    from .sort_direction import SortDirection
    from .source_of_payment_code import SourceOfPaymentCode
    from .state import State
    from .street_address_base import StreetAddressBase
    from .street_address_long_zip import StreetAddressLongZip
    from .street_address_long_zip_optional import StreetAddressLongZipOptional
    from .street_address_optional_base import StreetAddressOptionalBase
    from .street_address_short_zip import StreetAddressShortZip
    from .street_address_short_zip_optional import StreetAddressShortZipOptional
    from .task_assignment_id import TaskAssignmentId
    from .task_id import TaskId
    from .task_note_id import TaskNoteId
    from .unauthorized_error_message import UnauthorizedErrorMessage
    from .unprocessable_entity_error_message import UnprocessableEntityErrorMessage
    from .updates_disabled_due_to_external_system_integration_error_message import (
        UpdatesDisabledDueToExternalSystemIntegrationErrorMessage,
    )
    from .user_id import UserId
    from .work_queue_id import WorkQueueId
_dynamic_imports: typing.Dict[str, str] = {
    "AdjustmentId": ".adjustment_id",
    "AppointmentId": ".appointment_id",
    "BadRequestErrorMessage": ".bad_request_error_message",
    "BillingProviderCommercialLicenseType": ".billing_provider_commercial_license_type",
    "ChargeCaptureClaimCreationId": ".charge_capture_claim_creation_id",
    "ChargeCaptureId": ".charge_capture_id",
    "ChargeCapturePostBilledChangeId": ".charge_capture_post_billed_change_id",
    "ClaimAdjustmentGroupCodes": ".claim_adjustment_group_codes",
    "ClaimId": ".claim_id",
    "ClaimSubmissionPayerResponsibilityType": ".claim_submission_payer_responsibility_type",
    "Clearinghouse": ".clearinghouse",
    "ClinicalTrialId": ".clinical_trial_id",
    "Date": ".date",
    "DateRangeOptionalEnd": ".date_range_optional_end",
    "Decimal": ".decimal",
    "DelayReasonCode": ".delay_reason_code",
    "Email": ".email",
    "EmrPayerCrosswalk": ".emr_payer_crosswalk",
    "EncounterExternalId": ".encounter_external_id",
    "EncounterId": ".encounter_id",
    "EntityConflictErrorMessage": ".entity_conflict_error_message",
    "EntityNotFoundErrorMessage": ".entity_not_found_error_message",
    "EpsdtReferralConditionIndicatorCode": ".epsdt_referral_condition_indicator_code",
    "ErrorMessage": ".error_message",
    "FacilityTypeCode": ".facility_type_code",
    "InsuranceTypeCode": ".insurance_type_code",
    "IntendedSubmissionMedium": ".intended_submission_medium",
    "InternalErrorMessage": ".internal_error_message",
    "InvoiceId": ".invoice_id",
    "LinkUrl": ".link_url",
    "NetworkType": ".network_type",
    "NextResponsibleParty": ".next_responsible_party",
    "Npi": ".npi",
    "OrganizationId": ".organization_id",
    "OrganizationNotAuthorizedErrorMessage": ".organization_not_authorized_error_message",
    "PageToken": ".page_token",
    "PatientExternalId": ".patient_external_id",
    "PatientRelationshipToInsuredCodeAll": ".patient_relationship_to_insured_code_all",
    "PayerPlanGroupId": ".payer_plan_group_id",
    "PhoneNumber": ".phone_number",
    "PhoneNumberOptional": ".phone_number_optional",
    "PhoneNumberType": ".phone_number_type",
    "PreEncounterAppointmentId": ".pre_encounter_appointment_id",
    "PreEncounterPatientId": ".pre_encounter_patient_id",
    "Primitive": ".primitive",
    "ProcedureModifier": ".procedure_modifier",
    "ProviderCredentialingSpanId": ".provider_credentialing_span_id",
    "ProviderId": ".provider_id",
    "QualifierCode": ".qualifier_code",
    "RateId": ".rate_id",
    "RegionNational": ".region_national",
    "RegionStates": ".region_states",
    "Regions": ".regions",
    "Regions_National": ".regions",
    "Regions_States": ".regions",
    "RemovableDateRangeOptionalEnd": ".removable_date_range_optional_end",
    "RemovableDateRangeOptionalEnd_DateRange": ".removable_date_range_optional_end",
    "RemovableDateRangeOptionalEnd_Remove": ".removable_date_range_optional_end",
    "ReportTransmissionCode": ".report_transmission_code",
    "ReportTypeCode": ".report_type_code",
    "RequestValidationError": ".request_validation_error",
    "ResourcePage": ".resource_page",
    "SchemaId": ".schema_id",
    "ServiceLineId": ".service_line_id",
    "ServiceLineUnits": ".service_line_units",
    "SortDirection": ".sort_direction",
    "SourceOfPaymentCode": ".source_of_payment_code",
    "State": ".state",
    "StreetAddressBase": ".street_address_base",
    "StreetAddressLongZip": ".street_address_long_zip",
    "StreetAddressLongZipOptional": ".street_address_long_zip_optional",
    "StreetAddressOptionalBase": ".street_address_optional_base",
    "StreetAddressShortZip": ".street_address_short_zip",
    "StreetAddressShortZipOptional": ".street_address_short_zip_optional",
    "TaskAssignmentId": ".task_assignment_id",
    "TaskId": ".task_id",
    "TaskNoteId": ".task_note_id",
    "UnauthorizedErrorMessage": ".unauthorized_error_message",
    "UnprocessableEntityErrorMessage": ".unprocessable_entity_error_message",
    "UpdatesDisabledDueToExternalSystemIntegrationErrorMessage": ".updates_disabled_due_to_external_system_integration_error_message",
    "UserId": ".user_id",
    "WorkQueueId": ".work_queue_id",
}


def __getattr__(attr_name: str) -> typing.Any:
    module_name = _dynamic_imports.get(attr_name)
    if module_name is None:
        raise AttributeError(f"No {attr_name} found in _dynamic_imports for module name -> {__name__}")
    try:
        module = import_module(module_name, __package__)
        if module_name == f".{attr_name}":
            return module
        else:
            return getattr(module, attr_name)
    except ImportError as e:
        raise ImportError(f"Failed to import {attr_name} from {module_name}: {e}") from e
    except AttributeError as e:
        raise AttributeError(f"Failed to get {attr_name} from {module_name}: {e}") from e


def __dir__():
    lazy_attrs = list(_dynamic_imports.keys())
    return sorted(lazy_attrs)


__all__ = [
    "AdjustmentId",
    "AppointmentId",
    "BadRequestErrorMessage",
    "BillingProviderCommercialLicenseType",
    "ChargeCaptureClaimCreationId",
    "ChargeCaptureId",
    "ChargeCapturePostBilledChangeId",
    "ClaimAdjustmentGroupCodes",
    "ClaimId",
    "ClaimSubmissionPayerResponsibilityType",
    "Clearinghouse",
    "ClinicalTrialId",
    "Date",
    "DateRangeOptionalEnd",
    "Decimal",
    "DelayReasonCode",
    "Email",
    "EmrPayerCrosswalk",
    "EncounterExternalId",
    "EncounterId",
    "EntityConflictErrorMessage",
    "EntityNotFoundErrorMessage",
    "EpsdtReferralConditionIndicatorCode",
    "ErrorMessage",
    "FacilityTypeCode",
    "InsuranceTypeCode",
    "IntendedSubmissionMedium",
    "InternalErrorMessage",
    "InvoiceId",
    "LinkUrl",
    "NetworkType",
    "NextResponsibleParty",
    "Npi",
    "OrganizationId",
    "OrganizationNotAuthorizedErrorMessage",
    "PageToken",
    "PatientExternalId",
    "PatientRelationshipToInsuredCodeAll",
    "PayerPlanGroupId",
    "PhoneNumber",
    "PhoneNumberOptional",
    "PhoneNumberType",
    "PreEncounterAppointmentId",
    "PreEncounterPatientId",
    "Primitive",
    "ProcedureModifier",
    "ProviderCredentialingSpanId",
    "ProviderId",
    "QualifierCode",
    "RateId",
    "RegionNational",
    "RegionStates",
    "Regions",
    "Regions_National",
    "Regions_States",
    "RemovableDateRangeOptionalEnd",
    "RemovableDateRangeOptionalEnd_DateRange",
    "RemovableDateRangeOptionalEnd_Remove",
    "ReportTransmissionCode",
    "ReportTypeCode",
    "RequestValidationError",
    "ResourcePage",
    "SchemaId",
    "ServiceLineId",
    "ServiceLineUnits",
    "SortDirection",
    "SourceOfPaymentCode",
    "State",
    "StreetAddressBase",
    "StreetAddressLongZip",
    "StreetAddressLongZipOptional",
    "StreetAddressOptionalBase",
    "StreetAddressShortZip",
    "StreetAddressShortZipOptional",
    "TaskAssignmentId",
    "TaskId",
    "TaskNoteId",
    "UnauthorizedErrorMessage",
    "UnprocessableEntityErrorMessage",
    "UpdatesDisabledDueToExternalSystemIntegrationErrorMessage",
    "UserId",
    "WorkQueueId",
]



# This file was auto-generated by Fern from our API Definition.

import typing

from ........core import enum

T_Result = typing.TypeVar("T_Result")


class ReferralSource(enum.StrEnum):
    HOSPITAL = "HOSPITAL"
    REFERRING_MD = "REFERRING_MD"
    SELF = "SELF"
    OTHER = "OTHER"
    _UNKNOWN = "__REFERRALSOURCE_UNKNOWN__"
    """
    This member is used for forward compatibility. If the value is not recognized by the enum, it will be stored here, and the raw value is accessible through `.value`.
    """

    @classmethod
    def _missing_(cls, value: typing.Any) -> "ReferralSource":
        unknown = cls._UNKNOWN
        unknown._value_ = value
        return unknown

    def visit(
        self,
        hospital: typing.Callable[[], T_Result],
        referring_md: typing.Callable[[], T_Result],
        self_: typing.Callable[[], T_Result],
        other: typing.Callable[[], T_Result],
        _unknown_member: typing.Callable[[str], T_Result],
    ) -> T_Result:
        if self is ReferralSource.HOSPITAL:
            return hospital()
        if self is ReferralSource.REFERRING_MD:
            return referring_md()
        if self is ReferralSource.SELF:
            return self_()
        if self is ReferralSource.OTHER:
            return other()
        return _unknown_member(self._value_)



# This file was auto-generated by Fern from our API Definition.

import typing

from ......core import enum

T_Result = typing.TypeVar("T_Result")


class OccurrenceCode(enum.StrEnum):
    C_01 = "01"
    """
    Accident/Medical Coverage Code indicating accident-related injury for which there is medical payment coverage. Provide the date of accident/injury.
    """

    C_02 = "02"
    """
    No Fault Insurance Involved - Including Auto Accident/Other Code indicating the date of an accident including auto or other where state has applicable no fault liability laws (i.e., legal basis for settlement without admission of proof of guilt).
    """

    C_03 = "03"
    """
    Accident/Tort Liability Code indicating the date of an accident resulting from a third party’s action that may involve a civil court process in an attempt to require payment by the third party, other than no fault liability.
    """

    C_04 = "04"
    """
    Accident/Employment Related Code indicating the date of an accident allegedly relating to the patient’s employment.
    """

    C_05 = "05"
    """
    Accident/No Medical or Liability Coverage Code indicating accident related injury for which there is no medical payment or third-party liability coverage. Provide the date of accident/ injury.
    """

    C_06 = "06"
    """
    Crime Victim Code indicating the date on which a medical condition resulted from alleged criminal action committed by one or more parties.
    """

    C_09 = "09"
    """
    Start of Infertility Treatment Cycle Code indicating the date of start of infertility treatment cycle.
    """

    C_10 = "10"
    """
    Last Menstrual Period Code indicating the date of the last menstrual period; ONLY applies when patient is being treated for maternity related condition.
    """

    C_11 = "11"
    """
    Onset of Symptoms/Illness Code indicating the date the patient first became aware of symptoms/illness.
    """

    C_12 = "12"
    """
    Date of Onset for a Chronically Dependent Individual (HHA Claims Only.) Code denotes date the patient/beneficiary becomes a Chronically Dependent Individual (CDI). This is the first month of the 3-month period immediately before eligibility under respite care benefit.
    """

    C_16 = "16"
    """
    Date of Last Therapy Code denotes last day of therapy services (e.g., physical therapy, occupational therapy, speech therapy).
    """

    C_17 = "17"
    """
    Date Outpatient Occupational Therapy Plan Established or Last Reviewed Code denotes date an occupational therapy plan was established or last reviewed.
    """

    C_18 = "18"
    """
    Date of Retirement - Patient/ Beneficiary The date of retirement for the patient/ beneficiary.
    """

    C_19 = "19"
    """
    Date of Retirement - Spouse Code denotes the retirement date for the patient’s spouse.
    """

    C_20 = "20"
    """
    Date Guarantee of Payment Began Code indicates date on which the provider began claiming Medicare payment under the guarantee of payment provision (see Medicare manual for special Medicare instructions).
    """

    C_21 = "21"
    """
    Date UR Notice Received Code indicating the date of receipt by the provider of the UR Committee’s finding that the admission or future stay was not medically necessary.
    """

    C_22 = "22"
    """
    Date Active Care Ended Code indicates the date covered level of care ended in a SNF or general hospital, or date on which active care ended in a psychiatric or tuberculosis hospital, or date on which patient was released on a trial basis from a residential facility. Code not required when Condition Code 21 is used.
    """

    C_24 = "24"
    """
    Date Insurance Denied Code indicating the date the denial of coverage was received by the health care facility from any insurer.
    """

    C_25 = "25"
    """
    Date Benefits Terminated by Primary Payer Code indicating the date on which coverage (including Worker’s Compensation benefits or no-fault coverage) is no longer available to the patient.
    """

    C_26 = "26"
    """
    Date SNF Bed Became Available Code indicating the date on which a SNF bed became available to hospital inpatient who requires only SNF level care.
    """

    C_27 = "27"
    """
    Date of Hospice Certification or Recertification Code indicating the date of certification or recertification of the hospice benefit period.
    """

    C_28 = "28"
    """
    Date Comprehensive Outpatient Rehabilitation Plan Established or Last Reviewed Code indicating the date a comprehensive outpatient rehabilitation plan was established or last reviewed.
    """

    C_29 = "29"
    """
    Date Outpatient Physical Therapy Plan Established or Last Reviewed Code indicating the date a physical therapy plan established or last reviewed.
    """

    C_30 = "30"
    """
    Date Outpatient Speech Pathology Plan Established or Last Reviewed Code indicating the date a speech pathology plan was established or last reviewed.
    """

    C_31 = "31"
    """
    Date Beneficiary Notified of Intent to Bill (Accommodations) The date of notice provided by the hospital to the patient that inpatient care is no longer required.
    """

    C_32 = "32"
    """
    Date Beneficiary Notified of Intent to Bill (Procedures or Treatments) The date of notice provided to the beneficiary that requested care (diagnostic procedures or treatments) may not be reasonable or necessary.
    """

    C_33 = "33"
    """
    First Day of the Coordination Period for ESRD Beneficiaries Covered by EGHP Code indicates the first day of coordination for benefits that are secondary to benefits payable under an employer’s group health plan. Required only for ESRD beneficiaries.
    """

    C_34 = "34"
    """
    Date of Election of Extended Care Facilities Code indicates the date the guest elected to receive extended care services (used by Religious Non-Medical only).
    """

    C_35 = "35"
    """
    Date Treatment Started for Physical Therapy Code indicates the initial date services by the billing provider for physical therapy began.
    """

    C_36 = "36"
    """
    Date of Inpatient Hospital Discharge for Covered Transplant Patient Code indicates the date of discharge for inpatient hospital stay in which the patient received a covered transplant procedure when the hospital is billing for immunosuppressive drugs. Note: When the patient received both a covered and a non-covered transplant, the covered transplant predominates.
    """

    C_37 = "37"
    """
    Date of Inpatient Hospital Discharge for Non-covered Transplant Patient Code indicates the date of discharge for the inpatient hospital stay in which the patient received a non-covered transplant procedure when the hospital is billing for immunosuppressive drugs.
    """

    C_38 = "38"
    """
    Date Treatment Started for Home IV Therapy Date the patient was first treated at home for IV therapy. (Home IV providers - Bill Type 085x.)
    """

    C_39 = "39"
    """
    Date Discharged on a Continuous Course of IV Therapy Date the patient was discharged from the hospital on continuous course of IV therapy. (Home IV providers - Bill Type 085x.)
    """

    C_40 = "40"
    """
    SCHEDULED DATE OF ADMISSION - The scheduled date the patient will be admitted as an inpatient to the hospital. (This code may only be used on an outpatient claim.)
    """

    C_41 = "41"
    """
    DATE OF FIRST TEST PRE-ADMISSION TESTING - The date on which the first outpatient diagnostic test was performed as part of a PAT program. This code may only be used if a date of admission was scheduled before the administration of the test(s).
    """

    C_42 = "42"
    """
    DATE OF DISCHARGE - Use only when "Through" date in Form Locator 06 (Statement Covers Period) is not the actual discharge date and the frequency code in Form Locator 04 is that of a final bill. For final bill for hospice care, enter the date the Medicare beneficiary terminated his election of hospice care.
    """

    C_43 = "43"
    """
    SCHEDULED DATE OF CANCELED SURGERY - The date for which outpatient surgery was scheduled.
    """

    C_44 = "44"
    """
    DATE TREATMENT STARTED OCCUPATIONAL THERAPY - The date services were initiated by the billing provider for occupational therapy.
    """

    C_45 = "45"
    """
    DATE TREATMENT STARTED FOR SPEECH THERAPY - The date services were initiated by the billing provider for speech therapy.
    """

    C_46 = "46"
    """
    DATE TREATMENT STARTED FOR CARDIAC REHABILITATION - The date services were initiated by the billing provider for cardiac rehabilitation.
    """

    C_47 = "47"
    """
    DATE COST OUTLIER STATUS BEGINS - Code indicates that this is the first day after the day the Cost Outlier threshold is reached. For Medicare purposes, a beneficiary must have regular, coinsurance and/or lifetime reserve days available beginning on this date to allow coverage of additional daily charges for the purpose of making a cost outlier payment.
    """

    C_50 = "50"
    """
    ASSESSMENT DATE (EFFECTIVE 1/1/11) - Code indicating an assessment date as defined by the assessment instrument applicable to this provider type (e.g., Minimum Data Set (MDS) for skilled nursing).
    """

    C_51 = "51"
    """
    DATE OF LAST KT/V READING (EFFECTIVE 7/1/10) - For in-center hemodialysis patients, this is the date of the last reading taken during the billing period. For peritoneal dialysis patients (and home hemodialysis patients), this date may be before the current billing period but should be within 4 months of the date of service.
    """

    C_52 = "52"
    """
    MEDICAL CERTIFICATION/RECERTIFICATION DATE (EFFECTIVE 1/1/11) - The date of the most recent non-hospice medical certification or recertification of the patient. Use Occurrence Code 27 for Date of Hospice Certification or Recertification.
    """

    C_54 = "54"
    """
    PHYSICIAN FOLLOW-UP DATE (EFFECTIVE 1/1/11) - Last date of a physician follow-up with the patient.
    """

    C_55 = "55"
    """
    DATE OF DEATH (EFFECTIVE 10/1/12) - Report this code when patient discharge status codes 20 (expired), 40 (expired at home), 41 (expired in a medical facility), or 42 (expired - place unknown) are used.
    """

    C_56 = "56"
    """
    ORIGINAL HOSPICE ELECTION OR REVOCATION DATE (EFFECTIVE 1/1/18) - Used when the hospice is submitting a correction to the election date reported on a previous 08xA Type of Bill or the revocation date on a previous 08xB Type of Bill. The hospice reports the correct dates in FL06 - Statement Covers Period (From-Through) and the original date in this occurrence code.
    """

    C_61 = "61"
    """
    HOSPITAL DISCHARGE DATE (HHA ONLY) (EFFECTIVE 1/1/2020) - The Through date of a hospital stay that ended within 14 days prior to the From date of this HHA claim.
    """

    C_62 = "62"
    """
    OTHER INSTITUTIONAL DISCHARGE DATE (HHA ONLY) (EFFECTIVE 1/1/2020) - The Through date of a skilled nursing facility (SNF), inpatient rehabilitation facility (IRF), long term care hospital (LTCH), or inpatient psychiatric facility (IPF) stay that ended within 14 days prior to this HHA admission.
    """

    CA_1 = "A1"
    """
    BIRTH DATE - INSURED A - The birth date of the individual in whose name the insurance is carried.
    """

    CA_2 = "A2"
    """
    EFFECTIVE DATE - INSURED A POLICY - A code indicating the first date insurance is in force.
    """

    CA_3 = "A3"
    """
    BENEFITS EXHAUSTED - PAYER A - Code indicating the last date for which benefits are available and after which no payment can be made by Payer A.
    """

    CA_4 = "A4"
    """
    SPLIT BILL DATE - Date patient became eligible due to medically needy spend down (sometimes referred to as "Split Bill Date").
    """

    CB_1 = "B1"
    """
    BIRTH DATE INSURED B - The birth date of the individual in whose name the insurance is carried.
    """

    CB_2 = "B2"
    """
    EFFECTIVE DATE - INSURED B POLICY - A code indicating the first date insurance is in force.
    """

    CB_3 = "B3"
    """
    BENEFITS EXHAUSTED - PAYER B - Code indicating the last date for which benefits are available and after which no payment can be made by Payer B.
    """

    CC_1 = "C1"
    """
    BIRTH DATE INSURED C - The birth date of the individual in whose name the insurance is carried.
    """

    CC_2 = "C2"
    """
    EFFECTIVE DATE - INSURED C POLICY - A code indicating the first date insurance is in force.
    """

    CC_3 = "C3"
    """
    BENEFITS EXHAUSTED - PAYER C - Code indicating the last date for which benefits are available and after which no payment can be made by Payer C.
    """

    _UNKNOWN = "__OCCURRENCECODE_UNKNOWN__"
    """
    This member is used for forward compatibility. If the value is not recognized by the enum, it will be stored here, and the raw value is accessible through `.value`.
    """

    @classmethod
    def _missing_(cls, value: typing.Any) -> "OccurrenceCode":
        unknown = cls._UNKNOWN
        unknown._value_ = value
        return unknown

    def visit(
        self,
        c_01: typing.Callable[[], T_Result],
        c_02: typing.Callable[[], T_Result],
        c_03: typing.Callable[[], T_Result],
        c_04: typing.Callable[[], T_Result],
        c_05: typing.Callable[[], T_Result],
        c_06: typing.Callable[[], T_Result],
        c_09: typing.Callable[[], T_Result],
        c_10: typing.Callable[[], T_Result],
        c_11: typing.Callable[[], T_Result],
        c_12: typing.Callable[[], T_Result],
        c_16: typing.Callable[[], T_Result],
        c_17: typing.Callable[[], T_Result],
        c_18: typing.Callable[[], T_Result],
        c_19: typing.Callable[[], T_Result],
        c_20: typing.Callable[[], T_Result],
        c_21: typing.Callable[[], T_Result],
        c_22: typing.Callable[[], T_Result],
        c_24: typing.Callable[[], T_Result],
        c_25: typing.Callable[[], T_Result],
        c_26: typing.Callable[[], T_Result],
        c_27: typing.Callable[[], T_Result],
        c_28: typing.Callable[[], T_Result],
        c_29: typing.Callable[[], T_Result],
        c_30: typing.Callable[[], T_Result],
        c_31: typing.Callable[[], T_Result],
        c_32: typing.Callable[[], T_Result],
        c_33: typing.Callable[[], T_Result],
        c_34: typing.Callable[[], T_Result],
        c_35: typing.Callable[[], T_Result],
        c_36: typing.Callable[[], T_Result],
        c_37: typing.Callable[[], T_Result],
        c_38: typing.Callable[[], T_Result],
        c_39: typing.Callable[[], T_Result],
        c_40: typing.Callable[[], T_Result],
        c_41: typing.Callable[[], T_Result],
        c_42: typing.Callable[[], T_Result],
        c_43: typing.Callable[[], T_Result],
        c_44: typing.Callable[[], T_Result],
        c_45: typing.Callable[[], T_Result],
        c_46: typing.Callable[[], T_Result],
        c_47: typing.Callable[[], T_Result],
        c_50: typing.Callable[[], T_Result],
        c_51: typing.Callable[[], T_Result],
        c_52: typing.Callable[[], T_Result],
        c_54: typing.Callable[[], T_Result],
        c_55: typing.Callable[[], T_Result],
        c_56: typing.Callable[[], T_Result],
        c_61: typing.Callable[[], T_Result],
        c_62: typing.Callable[[], T_Result],
        ca_1: typing.Callable[[], T_Result],
        ca_2: typing.Callable[[], T_Result],
        ca_3: typing.Callable[[], T_Result],
        ca_4: typing.Callable[[], T_Result],
        cb_1: typing.Callable[[], T_Result],
        cb_2: typing.Callable[[], T_Result],
        cb_3: typing.Callable[[], T_Result],
        cc_1: typing.Callable[[], T_Result],
        cc_2: typing.Callable[[], T_Result],
        cc_3: typing.Callable[[], T_Result],
        _unknown_member: typing.Callable[[str], T_Result],
    ) -> T_Result:
        if self is OccurrenceCode.C_01:
            return c_01()
        if self is OccurrenceCode.C_02:
            return c_02()
        if self is OccurrenceCode.C_03:
            return c_03()
        if self is OccurrenceCode.C_04:
            return c_04()
        if self is OccurrenceCode.C_05:
            return c_05()
        if self is OccurrenceCode.C_06:
            return c_06()
        if self is OccurrenceCode.C_09:
            return c_09()
        if self is OccurrenceCode.C_10:
            return c_10()
        if self is OccurrenceCode.C_11:
            return c_11()
        if self is OccurrenceCode.C_12:
            return c_12()
        if self is OccurrenceCode.C_16:
            return c_16()
        if self is OccurrenceCode.C_17:
            return c_17()
        if self is OccurrenceCode.C_18:
            return c_18()
        if self is OccurrenceCode.C_19:
            return c_19()
        if self is OccurrenceCode.C_20:
            return c_20()
        if self is OccurrenceCode.C_21:
            return c_21()
        if self is OccurrenceCode.C_22:
            return c_22()
        if self is OccurrenceCode.C_24:
            return c_24()
        if self is OccurrenceCode.C_25:
            return c_25()
        if self is OccurrenceCode.C_26:
            return c_26()
        if self is OccurrenceCode.C_27:
            return c_27()
        if self is OccurrenceCode.C_28:
            return c_28()
        if self is OccurrenceCode.C_29:
            return c_29()
        if self is OccurrenceCode.C_30:
            return c_30()
        if self is OccurrenceCode.C_31:
            return c_31()
        if self is OccurrenceCode.C_32:
            return c_32()
        if self is OccurrenceCode.C_33:
            return c_33()
        if self is OccurrenceCode.C_34:
            return c_34()
        if self is OccurrenceCode.C_35:
            return c_35()
        if self is OccurrenceCode.C_36:
            return c_36()
        if self is OccurrenceCode.C_37:
            return c_37()
        if self is OccurrenceCode.C_38:
            return c_38()
        if self is OccurrenceCode.C_39:
            return c_39()
        if self is OccurrenceCode.C_40:
            return c_40()
        if self is OccurrenceCode.C_41:
            return c_41()
        if self is OccurrenceCode.C_42:
            return c_42()
        if self is OccurrenceCode.C_43:
            return c_43()
        if self is OccurrenceCode.C_44:
            return c_44()
        if self is OccurrenceCode.C_45:
            return c_45()
        if self is OccurrenceCode.C_46:
            return c_46()
        if self is OccurrenceCode.C_47:
            return c_47()
        if self is OccurrenceCode.C_50:
            return c_50()
        if self is OccurrenceCode.C_51:
            return c_51()
        if self is OccurrenceCode.C_52:
            return c_52()
        if self is OccurrenceCode.C_54:
            return c_54()
        if self is OccurrenceCode.C_55:
            return c_55()
        if self is OccurrenceCode.C_56:
            return c_56()
        if self is OccurrenceCode.C_61:
            return c_61()
        if self is OccurrenceCode.C_62:
            return c_62()
        if self is OccurrenceCode.CA_1:
            return ca_1()
        if self is OccurrenceCode.CA_2:
            return ca_2()
        if self is OccurrenceCode.CA_3:
            return ca_3()
        if self is OccurrenceCode.CA_4:
            return ca_4()
        if self is OccurrenceCode.CB_1:
            return cb_1()
        if self is OccurrenceCode.CB_2:
            return cb_2()
        if self is OccurrenceCode.CB_3:
            return cb_3()
        if self is OccurrenceCode.CC_1:
            return cc_1()
        if self is OccurrenceCode.CC_2:
            return cc_2()
        if self is OccurrenceCode.CC_3:
            return cc_3()
        return _unknown_member(self._value_)



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

import typing
from importlib import import_module

if typing.TYPE_CHECKING:
    from .create_write_offs_response import CreateWriteOffsResponse
    from .insurance_write_off import InsuranceWriteOff
    from .insurance_write_off_create import InsuranceWriteOffCreate
    from .insurance_write_off_reason import InsuranceWriteOffReason
    from .insurance_write_off_target import (
        InsuranceWriteOffTarget,
        InsuranceWriteOffTarget_BillingProviderId,
        InsuranceWriteOffTarget_ClaimId,
        InsuranceWriteOffTarget_ServiceLineId,
    )
    from .non_insurance_payer_write_off import NonInsurancePayerWriteOff
    from .non_insurance_payer_write_off_create import NonInsurancePayerWriteOffCreate
    from .patient_write_off import PatientWriteOff
    from .patient_write_off_create import PatientWriteOffCreate
    from .patient_write_off_reason import PatientWriteOffReason
    from .write_off import WriteOff, WriteOff_Insurance, WriteOff_NonInsurancePayer, WriteOff_Patient
    from .write_off_create import (
        WriteOffCreate,
        WriteOffCreate_Insurance,
        WriteOffCreate_NonInsurancePayer,
        WriteOffCreate_Patient,
    )
    from .write_off_id import WriteOffId
    from .write_off_sort_field import WriteOffSortField
    from .write_offs_page import WriteOffsPage
_dynamic_imports: typing.Dict[str, str] = {
    "CreateWriteOffsResponse": ".create_write_offs_response",
    "InsuranceWriteOff": ".insurance_write_off",
    "InsuranceWriteOffCreate": ".insurance_write_off_create",
    "InsuranceWriteOffReason": ".insurance_write_off_reason",
    "InsuranceWriteOffTarget": ".insurance_write_off_target",
    "InsuranceWriteOffTarget_BillingProviderId": ".insurance_write_off_target",
    "InsuranceWriteOffTarget_ClaimId": ".insurance_write_off_target",
    "InsuranceWriteOffTarget_ServiceLineId": ".insurance_write_off_target",
    "NonInsurancePayerWriteOff": ".non_insurance_payer_write_off",
    "NonInsurancePayerWriteOffCreate": ".non_insurance_payer_write_off_create",
    "PatientWriteOff": ".patient_write_off",
    "PatientWriteOffCreate": ".patient_write_off_create",
    "PatientWriteOffReason": ".patient_write_off_reason",
    "WriteOff": ".write_off",
    "WriteOffCreate": ".write_off_create",
    "WriteOffCreate_Insurance": ".write_off_create",
    "WriteOffCreate_NonInsurancePayer": ".write_off_create",
    "WriteOffCreate_Patient": ".write_off_create",
    "WriteOffId": ".write_off_id",
    "WriteOffSortField": ".write_off_sort_field",
    "WriteOff_Insurance": ".write_off",
    "WriteOff_NonInsurancePayer": ".write_off",
    "WriteOff_Patient": ".write_off",
    "WriteOffsPage": ".write_offs_page",
}


def __getattr__(attr_name: str) -> typing.Any:
    module_name = _dynamic_imports.get(attr_name)
    if module_name is None:
        raise AttributeError(f"No {attr_name} found in _dynamic_imports for module name -> {__name__}")
    try:
        module = import_module(module_name, __package__)
        if module_name == f".{attr_name}":
            return module
        else:
            return getattr(module, attr_name)
    except ImportError as e:
        raise ImportError(f"Failed to import {attr_name} from {module_name}: {e}") from e
    except AttributeError as e:
        raise AttributeError(f"Failed to get {attr_name} from {module_name}: {e}") from e


def __dir__():
    lazy_attrs = list(_dynamic_imports.keys())
    return sorted(lazy_attrs)


__all__ = [
    "CreateWriteOffsResponse",
    "InsuranceWriteOff",
    "InsuranceWriteOffCreate",
    "InsuranceWriteOffReason",
    "InsuranceWriteOffTarget",
    "InsuranceWriteOffTarget_BillingProviderId",
    "InsuranceWriteOffTarget_ClaimId",
    "InsuranceWriteOffTarget_ServiceLineId",
    "NonInsurancePayerWriteOff",
    "NonInsurancePayerWriteOffCreate",
    "PatientWriteOff",
    "PatientWriteOffCreate",
    "PatientWriteOffReason",
    "WriteOff",
    "WriteOffCreate",
    "WriteOffCreate_Insurance",
    "WriteOffCreate_NonInsurancePayer",
    "WriteOffCreate_Patient",
    "WriteOffId",
    "WriteOffSortField",
    "WriteOff_Insurance",
    "WriteOff_NonInsurancePayer",
    "WriteOff_Patient",
    "WriteOffsPage",
]



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

import typing
from importlib import import_module

if typing.TYPE_CHECKING:
    from .types import (
        BatchEligibilityResponse,
        CoordinationOfBenefitsRecommendation,
        CopayEstimationRecommendation,
        CopayEstimationRecommendationPayload,
        EligibilityCheck,
        EligibilityCheckError,
        EligibilityCheckErrorDetails,
        EligibilityCheckErrorSource,
        EligibilityCheckMetadata,
        EligibilityCheckPage,
        EligibilityCheckStatus,
        EligibilityRecommendation,
        EligibilityRecommendationPatientInfo,
        EligibilityRecommendationPayload,
        EligibilityRecommendationPayload_CoordinationOfBenefits,
        EligibilityRecommendationPayload_CopayEstimation,
        EligibilityRecommendationPayload_MedicareAdvantage,
        EligibilityRecommendationPayload_UserConfiguredPrompts,
        EligibilityRequest,
        EligibilityResponse,
        EligibilityStatus,
        Encounter,
        IndividualProvider,
        MedicareAdvantageRecommendation,
        MedicareAdvantageRecommendationPayload,
        OrganizationProvider,
        ParsedResponse,
        PayerSearchResponse,
        PostEligibilityRecommendationRequest,
        Provider,
        RequestCorrection,
        StediPayer,
        StediPayerItem,
        UserConfiguredPromptsRecommendation,
        UserConfiguredPromptsRecommendationPayload,
        UserConfiguredPromptsResult,
        Vote,
        VoteValue,
    )
_dynamic_imports: typing.Dict[str, str] = {
    "BatchEligibilityResponse": ".types",
    "CoordinationOfBenefitsRecommendation": ".types",
    "CopayEstimationRecommendation": ".types",
    "CopayEstimationRecommendationPayload": ".types",
    "EligibilityCheck": ".types",
    "EligibilityCheckError": ".types",
    "EligibilityCheckErrorDetails": ".types",
    "EligibilityCheckErrorSource": ".types",
    "EligibilityCheckMetadata": ".types",
    "EligibilityCheckPage": ".types",
    "EligibilityCheckStatus": ".types",
    "EligibilityRecommendation": ".types",
    "EligibilityRecommendationPatientInfo": ".types",
    "EligibilityRecommendationPayload": ".types",
    "EligibilityRecommendationPayload_CoordinationOfBenefits": ".types",
    "EligibilityRecommendationPayload_CopayEstimation": ".types",
    "EligibilityRecommendationPayload_MedicareAdvantage": ".types",
    "EligibilityRecommendationPayload_UserConfiguredPrompts": ".types",
    "EligibilityRequest": ".types",
    "EligibilityResponse": ".types",
    "EligibilityStatus": ".types",
    "Encounter": ".types",
    "IndividualProvider": ".types",
    "MedicareAdvantageRecommendation": ".types",
    "MedicareAdvantageRecommendationPayload": ".types",
    "OrganizationProvider": ".types",
    "ParsedResponse": ".types",
    "PayerSearchResponse": ".types",
    "PostEligibilityRecommendationRequest": ".types",
    "Provider": ".types",
    "RequestCorrection": ".types",
    "StediPayer": ".types",
    "StediPayerItem": ".types",
    "UserConfiguredPromptsRecommendation": ".types",
    "UserConfiguredPromptsRecommendationPayload": ".types",
    "UserConfiguredPromptsResult": ".types",
    "Vote": ".types",
    "VoteValue": ".types",
}


def __getattr__(attr_name: str) -> typing.Any:
    module_name = _dynamic_imports.get(attr_name)
    if module_name is None:
        raise AttributeError(f"No {attr_name} found in _dynamic_imports for module name -> {__name__}")
    try:
        module = import_module(module_name, __package__)
        if module_name == f".{attr_name}":
            return module
        else:
            return getattr(module, attr_name)
    except ImportError as e:
        raise ImportError(f"Failed to import {attr_name} from {module_name}: {e}") from e
    except AttributeError as e:
        raise AttributeError(f"Failed to get {attr_name} from {module_name}: {e}") from e


def __dir__():
    lazy_attrs = list(_dynamic_imports.keys())
    return sorted(lazy_attrs)


__all__ = [
    "BatchEligibilityResponse",
    "CoordinationOfBenefitsRecommendation",
    "CopayEstimationRecommendation",
    "CopayEstimationRecommendationPayload",
    "EligibilityCheck",
    "EligibilityCheckError",
    "EligibilityCheckErrorDetails",
    "EligibilityCheckErrorSource",
    "EligibilityCheckMetadata",
    "EligibilityCheckPage",
    "EligibilityCheckStatus",
    "EligibilityRecommendation",
    "EligibilityRecommendationPatientInfo",
    "EligibilityRecommendationPayload",
    "EligibilityRecommendationPayload_CoordinationOfBenefits",
    "EligibilityRecommendationPayload_CopayEstimation",
    "EligibilityRecommendationPayload_MedicareAdvantage",
    "EligibilityRecommendationPayload_UserConfiguredPrompts",
    "EligibilityRequest",
    "EligibilityResponse",
    "EligibilityStatus",
    "Encounter",
    "IndividualProvider",
    "MedicareAdvantageRecommendation",
    "MedicareAdvantageRecommendationPayload",
    "OrganizationProvider",
    "ParsedResponse",
    "PayerSearchResponse",
    "PostEligibilityRecommendationRequest",
    "Provider",
    "RequestCorrection",
    "StediPayer",
    "StediPayerItem",
    "UserConfiguredPromptsRecommendation",
    "UserConfiguredPromptsRecommendationPayload",
    "UserConfiguredPromptsResult",
    "Vote",
    "VoteValue",
]



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing
from json.decoder import JSONDecodeError

from .....core.api_error import ApiError
from .....core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from .....core.datetime_utils import serialize_datetime
from .....core.http_response import AsyncHttpResponse, HttpResponse
from .....core.jsonable_encoder import jsonable_encoder
from .....core.pydantic_utilities import parse_obj_as
from .....core.request_options import RequestOptions
from ....commons.errors.entity_not_found_error import EntityNotFoundError
from ....commons.errors.unauthorized_error import UnauthorizedError
from ....commons.types.claim_id import ClaimId
from ....commons.types.entity_not_found_error_message import EntityNotFoundErrorMessage
from ....commons.types.page_token import PageToken
from ....commons.types.unauthorized_error_message import UnauthorizedErrorMessage
from .errors.invalid_filters_error import InvalidFiltersError
from .types.invalid_filters_error_type import InvalidFiltersErrorType
from .types.invoice_itemization_response import InvoiceItemizationResponse
from .types.list_inventory_paged_response import ListInventoryPagedResponse


class RawV1Client:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def list_inventory(
        self,
        *,
        since: typing.Optional[dt.datetime] = None,
        limit: typing.Optional[int] = None,
        page_token: typing.Optional[PageToken] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[ListInventoryPagedResponse]:
        """
        Retrieve a list of inventory records based on the provided filters. Each inventory record provides the latest invoiceable status of the associated claim.
        The response is paginated, and the `page_token` can be used to retrieve subsequent pages. Initial requests should not include `page_token`.

        Parameters
        ----------
        since : typing.Optional[dt.datetime]
            Timestamp to filter records since, inclusive

        limit : typing.Optional[int]
            Maximum number of records to return, default is 100

        page_token : typing.Optional[PageToken]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[ListInventoryPagedResponse]
        """
        _response = self._client_wrapper.httpx_client.request(
            "api/patient-ar/v1/inventory",
            base_url=self._client_wrapper.get_environment().candid_api,
            method="GET",
            params={
                "since": serialize_datetime(since) if since is not None else None,
                "limit": limit,
                "page_token": page_token,
            },
            request_options=request_options,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                ListInventoryPagedResponse,
                parse_obj_as(
                    type_=ListInventoryPagedResponse,  # type: ignore
                    object_=_response_json,
                ),
            )
            return HttpResponse(response=_response, data=_data)
        if "errorName" in _response_json:
            if _response_json["errorName"] == "InvalidFiltersError":
                raise InvalidFiltersError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        InvalidFiltersErrorType,
                        parse_obj_as(
                            type_=InvalidFiltersErrorType,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "UnauthorizedError":
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        UnauthorizedErrorMessage,
                        parse_obj_as(
                            type_=UnauthorizedErrorMessage,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def itemize(
        self, claim_id: ClaimId, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[InvoiceItemizationResponse]:
        """
        Provides detailed itemization of invoice data for a specific claim.

        Parameters
        ----------
        claim_id : ClaimId

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[InvoiceItemizationResponse]
        """
        _response = self._client_wrapper.httpx_client.request(
            f"api/patient-ar/v1/invoice-itemization/{jsonable_encoder(claim_id)}",
            base_url=self._client_wrapper.get_environment().candid_api,
            method="GET",
            request_options=request_options,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                InvoiceItemizationResponse,
                parse_obj_as(
                    type_=InvoiceItemizationResponse,  # type: ignore
                    object_=_response_json,
                ),
            )
            return HttpResponse(response=_response, data=_data)
        if "errorName" in _response_json:
            if _response_json["errorName"] == "EntityNotFoundError":
                raise EntityNotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        EntityNotFoundErrorMessage,
                        parse_obj_as(
                            type_=EntityNotFoundErrorMessage,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "UnauthorizedError":
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        UnauthorizedErrorMessage,
                        parse_obj_as(
                            type_=UnauthorizedErrorMessage,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawV1Client:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def list_inventory(
        self,
        *,
        since: typing.Optional[dt.datetime] = None,
        limit: typing.Optional[int] = None,
        page_token: typing.Optional[PageToken] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[ListInventoryPagedResponse]:
        """
        Retrieve a list of inventory records based on the provided filters. Each inventory record provides the latest invoiceable status of the associated claim.
        The response is paginated, and the `page_token` can be used to retrieve subsequent pages. Initial requests should not include `page_token`.

        Parameters
        ----------
        since : typing.Optional[dt.datetime]
            Timestamp to filter records since, inclusive

        limit : typing.Optional[int]
            Maximum number of records to return, default is 100

        page_token : typing.Optional[PageToken]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[ListInventoryPagedResponse]
        """
        _response = await self._client_wrapper.httpx_client.request(
            "api/patient-ar/v1/inventory",
            base_url=self._client_wrapper.get_environment().candid_api,
            method="GET",
            params={
                "since": serialize_datetime(since) if since is not None else None,
                "limit": limit,
                "page_token": page_token,
            },
            request_options=request_options,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                ListInventoryPagedResponse,
                parse_obj_as(
                    type_=ListInventoryPagedResponse,  # type: ignore
                    object_=_response_json,
                ),
            )
            return AsyncHttpResponse(response=_response, data=_data)
        if "errorName" in _response_json:
            if _response_json["errorName"] == "InvalidFiltersError":
                raise InvalidFiltersError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        InvalidFiltersErrorType,
                        parse_obj_as(
                            type_=InvalidFiltersErrorType,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "UnauthorizedError":
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        UnauthorizedErrorMessage,
                        parse_obj_as(
                            type_=UnauthorizedErrorMessage,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def itemize(
        self, claim_id: ClaimId, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[InvoiceItemizationResponse]:
        """
        Provides detailed itemization of invoice data for a specific claim.

        Parameters
        ----------
        claim_id : ClaimId

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[InvoiceItemizationResponse]
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"api/patient-ar/v1/invoice-itemization/{jsonable_encoder(claim_id)}",
            base_url=self._client_wrapper.get_environment().candid_api,
            method="GET",
            request_options=request_options,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                InvoiceItemizationResponse,
                parse_obj_as(
                    type_=InvoiceItemizationResponse,  # type: ignore
                    object_=_response_json,
                ),
            )
            return AsyncHttpResponse(response=_response, data=_data)
        if "errorName" in _response_json:
            if _response_json["errorName"] == "EntityNotFoundError":
                raise EntityNotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        EntityNotFoundErrorMessage,
                        parse_obj_as(
                            type_=EntityNotFoundErrorMessage,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "UnauthorizedError":
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        UnauthorizedErrorMessage,
                        parse_obj_as(
                            type_=UnauthorizedErrorMessage,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing

from ....core import enum

T_Result = typing.TypeVar("T_Result")


class Gender(enum.StrEnum):
    MALE = "male"
    FEMALE = "female"
    OTHER = "other"
    NOT_GIVEN = "not_given"
    UNKNOWN = "unknown"
    _UNKNOWN = "__GENDER_UNKNOWN__"
    """
    This member is used for forward compatibility. If the value is not recognized by the enum, it will be stored here, and the raw value is accessible through `.value`.
    """

    @classmethod
    def _missing_(cls, value: typing.Any) -> "Gender":
        unknown = cls._UNKNOWN
        unknown._value_ = value
        return unknown

    def visit(
        self,
        male: typing.Callable[[], T_Result],
        female: typing.Callable[[], T_Result],
        other: typing.Callable[[], T_Result],
        not_given: typing.Callable[[], T_Result],
        unknown: typing.Callable[[], T_Result],
        _unknown_member: typing.Callable[[str], T_Result],
    ) -> T_Result:
        if self is Gender.MALE:
            return male()
        if self is Gender.FEMALE:
            return female()
        if self is Gender.OTHER:
            return other()
        if self is Gender.NOT_GIVEN:
            return not_given()
        if self is Gender.UNKNOWN:
            return unknown()
        return _unknown_member(self._value_)



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from .....core.api_error import ApiError
from .....core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from .....core.http_response import AsyncHttpResponse, HttpResponse
from .....core.jsonable_encoder import jsonable_encoder
from .....core.pydantic_utilities import parse_obj_as
from .....core.request_options import RequestOptions
from ....commons.errors.http_request_validations_error import HttpRequestValidationsError
from ....commons.types.encounter_id import EncounterId
from ....commons.types.request_validation_error import RequestValidationError
from .errors.encounter_has_existing_guarantor_error import EncounterHasExistingGuarantorError
from .types.encounter_has_existing_guarantor_error_type import EncounterHasExistingGuarantorErrorType
from .types.guarantor import Guarantor
from .types.guarantor_create import GuarantorCreate
from .types.guarantor_id import GuarantorId
from .types.guarantor_update import GuarantorUpdate

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawV1Client:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def create(
        self,
        encounter_id: EncounterId,
        *,
        request: GuarantorCreate,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[Guarantor]:
        """
        Creates a new guarantor and returns the newly created Guarantor object.

        Parameters
        ----------
        encounter_id : EncounterId

        request : GuarantorCreate

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[Guarantor]
        """
        _response = self._client_wrapper.httpx_client.request(
            f"api/guarantors/v1/{jsonable_encoder(encounter_id)}",
            base_url=self._client_wrapper.get_environment().candid_api,
            method="POST",
            json=request,
            request_options=request_options,
            omit=OMIT,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                Guarantor,
                parse_obj_as(
                    type_=Guarantor,  # type: ignore
                    object_=_response_json,
                ),
            )
            return HttpResponse(response=_response, data=_data)
        if "errorName" in _response_json:
            if _response_json["errorName"] == "EncounterHasExistingGuarantorError":
                raise EncounterHasExistingGuarantorError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        EncounterHasExistingGuarantorErrorType,
                        parse_obj_as(
                            type_=EncounterHasExistingGuarantorErrorType,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "HttpRequestValidationsError":
                raise HttpRequestValidationsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.List[RequestValidationError],
                        parse_obj_as(
                            type_=typing.List[RequestValidationError],  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def get(
        self, guarantor_id: GuarantorId, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[Guarantor]:
        """
        Retrieves a guarantor by its `guarantor_id`.

        Parameters
        ----------
        guarantor_id : GuarantorId

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[Guarantor]
        """
        _response = self._client_wrapper.httpx_client.request(
            f"api/guarantors/v1/{jsonable_encoder(guarantor_id)}",
            base_url=self._client_wrapper.get_environment().candid_api,
            method="GET",
            request_options=request_options,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                Guarantor,
                parse_obj_as(
                    type_=Guarantor,  # type: ignore
                    object_=_response_json,
                ),
            )
            return HttpResponse(response=_response, data=_data)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def update(
        self,
        guarantor_id: GuarantorId,
        *,
        request: GuarantorUpdate,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[Guarantor]:
        """
        Updates a guarantor by its `guarantor_id`.

        Parameters
        ----------
        guarantor_id : GuarantorId

        request : GuarantorUpdate

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[Guarantor]
        """
        _response = self._client_wrapper.httpx_client.request(
            f"api/guarantors/v1/{jsonable_encoder(guarantor_id)}",
            base_url=self._client_wrapper.get_environment().candid_api,
            method="PATCH",
            json=request,
            request_options=request_options,
            omit=OMIT,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                Guarantor,
                parse_obj_as(
                    type_=Guarantor,  # type: ignore
                    object_=_response_json,
                ),
            )
            return HttpResponse(response=_response, data=_data)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawV1Client:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def create(
        self,
        encounter_id: EncounterId,
        *,
        request: GuarantorCreate,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[Guarantor]:
        """
        Creates a new guarantor and returns the newly created Guarantor object.

        Parameters
        ----------
        encounter_id : EncounterId

        request : GuarantorCreate

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[Guarantor]
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"api/guarantors/v1/{jsonable_encoder(encounter_id)}",
            base_url=self._client_wrapper.get_environment().candid_api,
            method="POST",
            json=request,
            request_options=request_options,
            omit=OMIT,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                Guarantor,
                parse_obj_as(
                    type_=Guarantor,  # type: ignore
                    object_=_response_json,
                ),
            )
            return AsyncHttpResponse(response=_response, data=_data)
        if "errorName" in _response_json:
            if _response_json["errorName"] == "EncounterHasExistingGuarantorError":
                raise EncounterHasExistingGuarantorError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        EncounterHasExistingGuarantorErrorType,
                        parse_obj_as(
                            type_=EncounterHasExistingGuarantorErrorType,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "HttpRequestValidationsError":
                raise HttpRequestValidationsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.List[RequestValidationError],
                        parse_obj_as(
                            type_=typing.List[RequestValidationError],  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def get(
        self, guarantor_id: GuarantorId, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[Guarantor]:
        """
        Retrieves a guarantor by its `guarantor_id`.

        Parameters
        ----------
        guarantor_id : GuarantorId

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[Guarantor]
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"api/guarantors/v1/{jsonable_encoder(guarantor_id)}",
            base_url=self._client_wrapper.get_environment().candid_api,
            method="GET",
            request_options=request_options,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                Guarantor,
                parse_obj_as(
                    type_=Guarantor,  # type: ignore
                    object_=_response_json,
                ),
            )
            return AsyncHttpResponse(response=_response, data=_data)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def update(
        self,
        guarantor_id: GuarantorId,
        *,
        request: GuarantorUpdate,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[Guarantor]:
        """
        Updates a guarantor by its `guarantor_id`.

        Parameters
        ----------
        guarantor_id : GuarantorId

        request : GuarantorUpdate

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[Guarantor]
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"api/guarantors/v1/{jsonable_encoder(guarantor_id)}",
            base_url=self._client_wrapper.get_environment().candid_api,
            method="PATCH",
            json=request,
            request_options=request_options,
            omit=OMIT,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                Guarantor,
                parse_obj_as(
                    type_=Guarantor,  # type: ignore
                    object_=_response_json,
                ),
            )
            return AsyncHttpResponse(response=_response, data=_data)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing

from ........core import enum

T_Result = typing.TypeVar("T_Result")


class CoverageStatus(enum.StrEnum):
    """
    enum to represent the statuses defined at https://build.fhir.org/valueset-fm-status.html
    """

    ACTIVE = "ACTIVE"
    CANCELLED = "CANCELLED"
    DRAFT = "DRAFT"
    ENTERED_IN_ERROR = "ENTERED_IN_ERROR"
    _UNKNOWN = "__COVERAGESTATUS_UNKNOWN__"
    """
    This member is used for forward compatibility. If the value is not recognized by the enum, it will be stored here, and the raw value is accessible through `.value`.
    """

    @classmethod
    def _missing_(cls, value: typing.Any) -> "CoverageStatus":
        unknown = cls._UNKNOWN
        unknown._value_ = value
        return unknown

    def visit(
        self,
        active: typing.Callable[[], T_Result],
        cancelled: typing.Callable[[], T_Result],
        draft: typing.Callable[[], T_Result],
        entered_in_error: typing.Callable[[], T_Result],
        _unknown_member: typing.Callable[[str], T_Result],
    ) -> T_Result:
        if self is CoverageStatus.ACTIVE:
            return active()
        if self is CoverageStatus.CANCELLED:
            return cancelled()
        if self is CoverageStatus.DRAFT:
            return draft()
        if self is CoverageStatus.ENTERED_IN_ERROR:
            return entered_in_error()
        return _unknown_member(self._value_)



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ......core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class AuthGetTokenResponse(UniversalBaseModel):
    """
    Examples
    --------
    from candid.resources.auth.resources.default import AuthGetTokenResponse

    AuthGetTokenResponse(
        access_token="eyJz93a...k4laUWw",
        expires_in=86400,
        token_type="Bearer",
    )
    """

    access_token: str
    expires_in: int = pydantic.Field()
    """
    Time in seconds.
    """

    token_type: str

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
from ......core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .....commons.types.charge_capture_claim_creation_id import ChargeCaptureClaimCreationId
from .....commons.types.charge_capture_id import ChargeCaptureId
from .....commons.types.patient_external_id import PatientExternalId
from .charge_capture_data import ChargeCaptureData
from .charge_capture_error import ChargeCaptureError
from .charge_capture_post_billed_change import ChargeCapturePostBilledChange
from .charge_capture_status import ChargeCaptureStatus


class ChargeCapture(UniversalBaseModel):
    id: ChargeCaptureId
    status: ChargeCaptureStatus
    charge_capture_data: ChargeCaptureData
    date_of_service: typing.Optional[dt.date] = None
    patient_external_id: PatientExternalId
    charge_external_id: str
    ehr_source_url: typing.Optional[str] = None
    originating_system: typing.Optional[str] = None
    claim_creation_category: typing.Optional[str] = None
    error: typing.Optional[ChargeCaptureError] = None
    updates: typing.List[ChargeCapturePostBilledChange] = pydantic.Field()
    """
    This list of updates will always return at most 1 update that is not resolved. The singular update will contain the difference between the updated charge and the created encounter.
    """

    claim_creation_id: typing.Optional[ChargeCaptureClaimCreationId] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ......core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .....invoices.resources.v_2.types.invoice_item_create import InvoiceItemCreate
from .invoice_item_update_type import InvoiceItemUpdateType


class InvoiceItemInfoUpdate(UniversalBaseModel):
    update_type: InvoiceItemUpdateType = pydantic.Field()
    """
    The only supported update operations for invoice items is to either overwrite the entire list of invoice items
    or to append new invoice items
    """

    items: typing.List[InvoiceItemCreate]

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

import typing
from importlib import import_module

if typing.TYPE_CHECKING:
    from . import appointments, common, coverages, eligibility_checks, images, lists, notes, patients, tags
    from .common import (
        AdditionalPayerInformation,
        Address,
        AddressUse,
        AppointmentId,
        BadRequestError,
        BaseModel,
        CanonicalClinicalTrialAssociation,
        CanonicalClinicalTrialId,
        CanonicalNonInsurancePayerAssociation,
        CanonicalNonInsurancePayerId,
        CanonicalProviderId,
        CanonicalServiceFacilityId,
        ContactPoint,
        ContactPointUse,
        CoverageId,
        DisabilityStatus,
        ErrorBase4Xx,
        Ethnicity,
        ExternalIdentifier,
        ExternalProvider,
        ExternalProviderType,
        FilterQueryString,
        Gender,
        HumanName,
        NameUse,
        NotFoundError,
        NoteId,
        OrganizationId,
        PageToken,
        PatientId,
        PatientServiceFacility,
        PayerId,
        PayerPlanGroupId,
        Period,
        Race,
        Relationship,
        ResourcePage,
        Sex,
        SexualOrientation,
        SortDirection,
        TagId,
        UserId,
        VersionConflictError,
        VersionConflictErrorBody,
    )
_dynamic_imports: typing.Dict[str, str] = {
    "AdditionalPayerInformation": ".common",
    "Address": ".common",
    "AddressUse": ".common",
    "AppointmentId": ".common",
    "BadRequestError": ".common",
    "BaseModel": ".common",
    "CanonicalClinicalTrialAssociation": ".common",
    "CanonicalClinicalTrialId": ".common",
    "CanonicalNonInsurancePayerAssociation": ".common",
    "CanonicalNonInsurancePayerId": ".common",
    "CanonicalProviderId": ".common",
    "CanonicalServiceFacilityId": ".common",
    "ContactPoint": ".common",
    "ContactPointUse": ".common",
    "CoverageId": ".common",
    "DisabilityStatus": ".common",
    "ErrorBase4Xx": ".common",
    "Ethnicity": ".common",
    "ExternalIdentifier": ".common",
    "ExternalProvider": ".common",
    "ExternalProviderType": ".common",
    "FilterQueryString": ".common",
    "Gender": ".common",
    "HumanName": ".common",
    "NameUse": ".common",
    "NotFoundError": ".common",
    "NoteId": ".common",
    "OrganizationId": ".common",
    "PageToken": ".common",
    "PatientId": ".common",
    "PatientServiceFacility": ".common",
    "PayerId": ".common",
    "PayerPlanGroupId": ".common",
    "Period": ".common",
    "Race": ".common",
    "Relationship": ".common",
    "ResourcePage": ".common",
    "Sex": ".common",
    "SexualOrientation": ".common",
    "SortDirection": ".common",
    "TagId": ".common",
    "UserId": ".common",
    "VersionConflictError": ".common",
    "VersionConflictErrorBody": ".common",
    "appointments": ".appointments",
    "common": ".common",
    "coverages": ".coverages",
    "eligibility_checks": ".eligibility_checks",
    "images": ".images",
    "lists": ".lists",
    "notes": ".notes",
    "patients": ".patients",
    "tags": ".tags",
}


def __getattr__(attr_name: str) -> typing.Any:
    module_name = _dynamic_imports.get(attr_name)
    if module_name is None:
        raise AttributeError(f"No {attr_name} found in _dynamic_imports for module name -> {__name__}")
    try:
        module = import_module(module_name, __package__)
        if module_name == f".{attr_name}":
            return module
        else:
            return getattr(module, attr_name)
    except ImportError as e:
        raise ImportError(f"Failed to import {attr_name} from {module_name}: {e}") from e
    except AttributeError as e:
        raise AttributeError(f"Failed to get {attr_name} from {module_name}: {e}") from e


def __dir__():
    lazy_attrs = list(_dynamic_imports.keys())
    return sorted(lazy_attrs)


__all__ = [
    "AdditionalPayerInformation",
    "Address",
    "AddressUse",
    "AppointmentId",
    "BadRequestError",
    "BaseModel",
    "CanonicalClinicalTrialAssociation",
    "CanonicalClinicalTrialId",
    "CanonicalNonInsurancePayerAssociation",
    "CanonicalNonInsurancePayerId",
    "CanonicalProviderId",
    "CanonicalServiceFacilityId",
    "ContactPoint",
    "ContactPointUse",
    "CoverageId",
    "DisabilityStatus",
    "ErrorBase4Xx",
    "Ethnicity",
    "ExternalIdentifier",
    "ExternalProvider",
    "ExternalProviderType",
    "FilterQueryString",
    "Gender",
    "HumanName",
    "NameUse",
    "NotFoundError",
    "NoteId",
    "OrganizationId",
    "PageToken",
    "PatientId",
    "PatientServiceFacility",
    "PayerId",
    "PayerPlanGroupId",
    "Period",
    "Race",
    "Relationship",
    "ResourcePage",
    "Sex",
    "SexualOrientation",
    "SortDirection",
    "TagId",
    "UserId",
    "VersionConflictError",
    "VersionConflictErrorBody",
    "appointments",
    "common",
    "coverages",
    "eligibility_checks",
    "images",
    "lists",
    "notes",
    "patients",
    "tags",
]



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

import typing
from importlib import import_module

if typing.TYPE_CHECKING:
    from .address import Address
    from .benefit_type import BenefitType
    from .benefits_related_entity import BenefitsRelatedEntity
    from .coverage import Coverage
    from .coverage_benefits import CoverageBenefits
    from .coverage_details import CoverageDetails
    from .coverage_eligibility_check_response import CoverageEligibilityCheckResponse
    from .coverage_level import CoverageLevel
    from .coverage_status import CoverageStatus
    from .coverage_value import CoverageValue
    from .coverage_value_unit import CoverageValueUnit
    from .coverages_page import CoveragesPage
    from .expanded_member_info import ExpandedMemberInfo
    from .insurance_plan import InsurancePlan
    from .insurance_type_code import InsuranceTypeCode
    from .latest_eligibility_check import LatestEligibilityCheck
    from .member_info import MemberInfo
    from .mutable_coverage import MutableCoverage
    from .network_type import NetworkType
    from .payer_plan_group_fields import PayerPlanGroupFields
    from .plan_coverage import PlanCoverage
    from .plan_coverage_details import PlanCoverageDetails
    from .plan_date import PlanDate
    from .plan_metadata import PlanMetadata
    from .related_entity_contact import RelatedEntityContact
    from .service_coverage import ServiceCoverage
    from .service_coverage_details import ServiceCoverageDetails
    from .service_type_code import ServiceTypeCode
    from .subscriber import Subscriber
_dynamic_imports: typing.Dict[str, str] = {
    "Address": ".address",
    "BenefitType": ".benefit_type",
    "BenefitsRelatedEntity": ".benefits_related_entity",
    "Coverage": ".coverage",
    "CoverageBenefits": ".coverage_benefits",
    "CoverageDetails": ".coverage_details",
    "CoverageEligibilityCheckResponse": ".coverage_eligibility_check_response",
    "CoverageLevel": ".coverage_level",
    "CoverageStatus": ".coverage_status",
    "CoverageValue": ".coverage_value",
    "CoverageValueUnit": ".coverage_value_unit",
    "CoveragesPage": ".coverages_page",
    "ExpandedMemberInfo": ".expanded_member_info",
    "InsurancePlan": ".insurance_plan",
    "InsuranceTypeCode": ".insurance_type_code",
    "LatestEligibilityCheck": ".latest_eligibility_check",
    "MemberInfo": ".member_info",
    "MutableCoverage": ".mutable_coverage",
    "NetworkType": ".network_type",
    "PayerPlanGroupFields": ".payer_plan_group_fields",
    "PlanCoverage": ".plan_coverage",
    "PlanCoverageDetails": ".plan_coverage_details",
    "PlanDate": ".plan_date",
    "PlanMetadata": ".plan_metadata",
    "RelatedEntityContact": ".related_entity_contact",
    "ServiceCoverage": ".service_coverage",
    "ServiceCoverageDetails": ".service_coverage_details",
    "ServiceTypeCode": ".service_type_code",
    "Subscriber": ".subscriber",
}


def __getattr__(attr_name: str) -> typing.Any:
    module_name = _dynamic_imports.get(attr_name)
    if module_name is None:
        raise AttributeError(f"No {attr_name} found in _dynamic_imports for module name -> {__name__}")
    try:
        module = import_module(module_name, __package__)
        if module_name == f".{attr_name}":
            return module
        else:
            return getattr(module, attr_name)
    except ImportError as e:
        raise ImportError(f"Failed to import {attr_name} from {module_name}: {e}") from e
    except AttributeError as e:
        raise AttributeError(f"Failed to get {attr_name} from {module_name}: {e}") from e


def __dir__():
    lazy_attrs = list(_dynamic_imports.keys())
    return sorted(lazy_attrs)


__all__ = [
    "Address",
    "BenefitType",
    "BenefitsRelatedEntity",
    "Coverage",
    "CoverageBenefits",
    "CoverageDetails",
    "CoverageEligibilityCheckResponse",
    "CoverageLevel",
    "CoverageStatus",
    "CoverageValue",
    "CoverageValueUnit",
    "CoveragesPage",
    "ExpandedMemberInfo",
    "InsurancePlan",
    "InsuranceTypeCode",
    "LatestEligibilityCheck",
    "MemberInfo",
    "MutableCoverage",
    "NetworkType",
    "PayerPlanGroupFields",
    "PlanCoverage",
    "PlanCoverageDetails",
    "PlanDate",
    "PlanMetadata",
    "RelatedEntityContact",
    "ServiceCoverage",
    "ServiceCoverageDetails",
    "ServiceTypeCode",
    "Subscriber",
]



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from .....core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from .....core.request_options import RequestOptions
from ....commons.types.claim_id import ClaimId
from ....commons.types.page_token import PageToken
from .raw_client import AsyncRawV1Client, RawV1Client
from .types.invoice_itemization_response import InvoiceItemizationResponse
from .types.list_inventory_paged_response import ListInventoryPagedResponse


class V1Client:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawV1Client(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawV1Client:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawV1Client
        """
        return self._raw_client

    def list_inventory(
        self,
        *,
        since: typing.Optional[dt.datetime] = None,
        limit: typing.Optional[int] = None,
        page_token: typing.Optional[PageToken] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ListInventoryPagedResponse:
        """
        Retrieve a list of inventory records based on the provided filters. Each inventory record provides the latest invoiceable status of the associated claim.
        The response is paginated, and the `page_token` can be used to retrieve subsequent pages. Initial requests should not include `page_token`.

        Parameters
        ----------
        since : typing.Optional[dt.datetime]
            Timestamp to filter records since, inclusive

        limit : typing.Optional[int]
            Maximum number of records to return, default is 100

        page_token : typing.Optional[PageToken]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ListInventoryPagedResponse

        Examples
        --------
        from candid import CandidApiClient

        client = CandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        client.patient_ar.v_1.list_inventory()
        """
        _response = self._raw_client.list_inventory(
            since=since, limit=limit, page_token=page_token, request_options=request_options
        )
        return _response.data

    def itemize(
        self, claim_id: ClaimId, *, request_options: typing.Optional[RequestOptions] = None
    ) -> InvoiceItemizationResponse:
        """
        Provides detailed itemization of invoice data for a specific claim.

        Parameters
        ----------
        claim_id : ClaimId

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        InvoiceItemizationResponse

        Examples
        --------
        import uuid

        from candid import CandidApiClient

        client = CandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        client.patient_ar.v_1.itemize(
            claim_id=uuid.UUID(
                "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
            ),
        )
        """
        _response = self._raw_client.itemize(claim_id, request_options=request_options)
        return _response.data


class AsyncV1Client:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawV1Client(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawV1Client:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawV1Client
        """
        return self._raw_client

    async def list_inventory(
        self,
        *,
        since: typing.Optional[dt.datetime] = None,
        limit: typing.Optional[int] = None,
        page_token: typing.Optional[PageToken] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ListInventoryPagedResponse:
        """
        Retrieve a list of inventory records based on the provided filters. Each inventory record provides the latest invoiceable status of the associated claim.
        The response is paginated, and the `page_token` can be used to retrieve subsequent pages. Initial requests should not include `page_token`.

        Parameters
        ----------
        since : typing.Optional[dt.datetime]
            Timestamp to filter records since, inclusive

        limit : typing.Optional[int]
            Maximum number of records to return, default is 100

        page_token : typing.Optional[PageToken]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ListInventoryPagedResponse

        Examples
        --------
        import asyncio

        from candid import AsyncCandidApiClient

        client = AsyncCandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )


        async def main() -> None:
            await client.patient_ar.v_1.list_inventory()


        asyncio.run(main())
        """
        _response = await self._raw_client.list_inventory(
            since=since, limit=limit, page_token=page_token, request_options=request_options
        )
        return _response.data

    async def itemize(
        self, claim_id: ClaimId, *, request_options: typing.Optional[RequestOptions] = None
    ) -> InvoiceItemizationResponse:
        """
        Provides detailed itemization of invoice data for a specific claim.

        Parameters
        ----------
        claim_id : ClaimId

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        InvoiceItemizationResponse

        Examples
        --------
        import asyncio
        import uuid

        from candid import AsyncCandidApiClient

        client = AsyncCandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )


        async def main() -> None:
            await client.patient_ar.v_1.itemize(
                claim_id=uuid.UUID(
                    "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
                ),
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.itemize(claim_id, request_options=request_options)
        return _response.data
